"""Common operations on Posix pathnames.

Instead of importing this module directly, import os and refer to
this module as os.path.  The "os.path" name is an alias for this
module on Posix systems; on other systems (e.g. Windows),
os.path provides the same operations in a manner specific to that
platform, and is an alias to another module (e.g. ntpath).

Some of this can actually be useful on non-Posix systems too, e.g.
for manipulation of the pathname component of URLs.
"""

import sys
from _typeshed import AnyOrLiteralStr, BytesPath, FileDescriptorOrPath, StrOrBytesPath, StrPath
from collections.abc import Iterable
from genericpath import (
    ALLOW_MISSING as ALLOW_MISSING,
    _AllowMissingType,
    commonprefix as commonprefix,
    exists as exists,
    getatime as getatime,
    getctime as getctime,
    getmtime as getmtime,
    getsize as getsize,
    isdir as isdir,
    isfile as isfile,
    samefile as samefile,
    sameopenfile as sameopenfile,
    samestat as samestat,
)

if sys.version_info >= (3, 13):
    from genericpath import isdevdrive as isdevdrive
from os import PathLike
from typing import AnyStr, overload
from typing_extensions import LiteralString

__all__ = [
    "normcase",
    "isabs",
    "join",
    "splitdrive",
    "split",
    "splitext",
    "basename",
    "dirname",
    "commonprefix",
    "getsize",
    "getmtime",
    "getatime",
    "getctime",
    "islink",
    "exists",
    "lexists",
    "isdir",
    "isfile",
    "ismount",
    "expanduser",
    "expandvars",
    "normpath",
    "abspath",
    "samefile",
    "sameopenfile",
    "samestat",
    "curdir",
    "pardir",
    "sep",
    "pathsep",
    "defpath",
    "altsep",
    "extsep",
    "devnull",
    "realpath",
    "supports_unicode_filenames",
    "relpath",
    "commonpath",
]
__all__ += ["ALLOW_MISSING"]
if sys.version_info >= (3, 12):
    __all__ += ["isjunction", "splitroot"]
if sys.version_info >= (3, 13):
    __all__ += ["isdevdrive"]

supports_unicode_filenames: bool
# aliases (also in os)
curdir: LiteralString
pardir: LiteralString
sep: LiteralString
altsep: LiteralString | None
extsep: LiteralString
pathsep: LiteralString
defpath: LiteralString
devnull: LiteralString

# Overloads are necessary to work around python/mypy#17952 & python/mypy#11880
@overload
def abspath(path: PathLike[AnyStr]) -> AnyStr:
    """Return an absolute path."""

@overload
def abspath(path: AnyStr) -> AnyStr: ...
@overload
def basename(p: PathLike[AnyStr]) -> AnyStr:
    """Returns the final component of a pathname"""

@overload
def basename(p: AnyOrLiteralStr) -> AnyOrLiteralStr: ...
@overload
def dirname(p: PathLike[AnyStr]) -> AnyStr:
    """Returns the directory component of a pathname"""

@overload
def dirname(p: AnyOrLiteralStr) -> AnyOrLiteralStr: ...
@overload
def expanduser(path: PathLike[AnyStr]) -> AnyStr:
    """Expand ~ and ~user constructions.  If user or $HOME is unknown,
    do nothing.
    """

@overload
def expanduser(path: AnyStr) -> AnyStr: ...
@overload
def expandvars(path: PathLike[AnyStr]) -> AnyStr:
    """Expand shell variables of form $var and ${var}.  Unknown variables
    are left unchanged.
    """

@overload
def expandvars(path: AnyStr) -> AnyStr: ...
@overload
def normcase(s: PathLike[AnyStr]) -> AnyStr:
    """Normalize case of pathname.  Has no effect under Posix"""

@overload
def normcase(s: AnyOrLiteralStr) -> AnyOrLiteralStr: ...
@overload
def normpath(path: PathLike[AnyStr]) -> AnyStr:
    """Normalize path, eliminating double slashes, etc."""

@overload
def normpath(path: AnyOrLiteralStr) -> AnyOrLiteralStr: ...
@overload
def commonpath(paths: Iterable[LiteralString]) -> LiteralString:
    """Given a sequence of path names, returns the longest common sub-path."""

@overload
def commonpath(paths: Iterable[StrPath]) -> str: ...
@overload
def commonpath(paths: Iterable[BytesPath]) -> bytes: ...

# First parameter is not actually pos-only,
# but must be defined as pos-only in the stub or cross-platform code doesn't type-check,
# as the parameter name is different in ntpath.join()
@overload
def join(a: LiteralString, /, *paths: LiteralString) -> LiteralString:
    """Join two or more pathname components, inserting '/' as needed.
    If any component is an absolute path, all previous path components
    will be discarded.  An empty last part will result in a path that
    ends with a separator.
    """

@overload
def join(a: StrPath, /, *paths: StrPath) -> str: ...
@overload
def join(a: BytesPath, /, *paths: BytesPath) -> bytes: ...
@overload
def realpath(filename: PathLike[AnyStr], *, strict: bool | _AllowMissingType = False) -> AnyStr:
    """Return the canonical path of the specified filename, eliminating any
    symbolic links encountered in the path.
    """

@overload
def realpath(filename: AnyStr, *, strict: bool | _AllowMissingType = False) -> AnyStr: ...
@overload
def relpath(path: LiteralString, start: LiteralString | None = None) -> LiteralString:
    """Return a relative version of a path"""

@overload
def relpath(path: BytesPath, start: BytesPath | None = None) -> bytes: ...
@overload
def relpath(path: StrPath, start: StrPath | None = None) -> str: ...
@overload
def split(p: PathLike[AnyStr]) -> tuple[AnyStr, AnyStr]:
    """Split a pathname.  Returns tuple "(head, tail)" where "tail" is
    everything after the final slash.  Either part may be empty.
    """

@overload
def split(p: AnyOrLiteralStr) -> tuple[AnyOrLiteralStr, AnyOrLiteralStr]: ...
@overload
def splitdrive(p: PathLike[AnyStr]) -> tuple[AnyStr, AnyStr]:
    """Split a pathname into drive and path. On Posix, drive is always
    empty.
    """

@overload
def splitdrive(p: AnyOrLiteralStr) -> tuple[AnyOrLiteralStr, AnyOrLiteralStr]: ...
@overload
def splitext(p: PathLike[AnyStr]) -> tuple[AnyStr, AnyStr]:
    """Split the extension from a pathname.

    Extension is everything from the last dot to the end, ignoring
    leading dots.  Returns "(root, ext)"; ext may be empty.
    """

@overload
def splitext(p: AnyOrLiteralStr) -> tuple[AnyOrLiteralStr, AnyOrLiteralStr]: ...
def isabs(s: StrOrBytesPath) -> bool:
    """Test whether a path is absolute"""

def islink(path: FileDescriptorOrPath) -> bool:
    """Test whether a path is a symbolic link"""

def ismount(path: FileDescriptorOrPath) -> bool:
    """Test whether a path is a mount point"""

def lexists(path: FileDescriptorOrPath) -> bool:
    """Test whether a path exists.  Returns True for broken symbolic links"""

if sys.version_info >= (3, 12):
    def isjunction(path: StrOrBytesPath) -> bool:
        """Test whether a path is a junction
        Junctions are not supported on the current platform
        """

    @overload
    def splitroot(p: AnyOrLiteralStr) -> tuple[AnyOrLiteralStr, AnyOrLiteralStr, AnyOrLiteralStr]:
        """Split a pathname into drive, root and tail.

        The tail contains anything after the root.
        """

    @overload
    def splitroot(p: PathLike[AnyStr]) -> tuple[AnyStr, AnyStr, AnyStr]: ...



"""A parser of RFC 2822 and MIME email messages."""

from _typeshed import SupportsRead
from collections.abc import Callable
from email._policybase import _MessageT
from email.feedparser import BytesFeedParser as BytesFeedParser, FeedParser as FeedParser
from email.message import Message
from email.policy import Policy
from io import _WrappedBuffer
from typing import Generic, overload

__all__ = ["Parser", "HeaderParser", "BytesParser", "BytesHeaderParser", "FeedParser", "BytesFeedParser"]

class Parser(Generic[_MessageT]):
    @overload
    def __init__(self: Parser[Message[str, str]], _class: None = None) -> None:
        """Parser of RFC 2822 and MIME email messages.

        Creates an in-memory object tree representing the email message, which
        can then be manipulated and turned over to a Generator to return the
        textual representation of the message.

        The string must be formatted as a block of RFC 2822 headers and header
        continuation lines, optionally preceded by a 'Unix-from' header.  The
        header block is terminated either by the end of the string or by a
        blank line.

        _class is the class to instantiate for new message objects when they
        must be created.  This class must have a constructor that can take
        zero arguments.  Default is Message.Message.

        The policy keyword specifies a policy object that controls a number of
        aspects of the parser's operation.  The default policy maintains
        backward compatibility.

        """

    @overload
    def __init__(self, _class: None = None, *, policy: Policy[_MessageT]) -> None: ...
    @overload
    def __init__(self, _class: Callable[[], _MessageT] | None, *, policy: Policy[_MessageT] = ...) -> None: ...
    def parse(self, fp: SupportsRead[str], headersonly: bool = False) -> _MessageT:
        """Create a message structure from the data in a file.

        Reads all the data from the file and returns the root of the message
        structure.  Optional headersonly is a flag specifying whether to stop
        parsing after reading the headers or not.  The default is False,
        meaning it parses the entire contents of the file.
        """

    def parsestr(self, text: str, headersonly: bool = False) -> _MessageT:
        """Create a message structure from a string.

        Returns the root of the message structure.  Optional headersonly is a
        flag specifying whether to stop parsing after reading the headers or
        not.  The default is False, meaning it parses the entire contents of
        the file.
        """

class HeaderParser(Parser[_MessageT]):
    def parse(self, fp: SupportsRead[str], headersonly: bool = True) -> _MessageT: ...
    def parsestr(self, text: str, headersonly: bool = True) -> _MessageT: ...

class BytesParser(Generic[_MessageT]):
    parser: Parser[_MessageT]
    @overload
    def __init__(self: BytesParser[Message[str, str]], _class: None = None) -> None:
        """Parser of binary RFC 2822 and MIME email messages.

        Creates an in-memory object tree representing the email message, which
        can then be manipulated and turned over to a Generator to return the
        textual representation of the message.

        The input must be formatted as a block of RFC 2822 headers and header
        continuation lines, optionally preceded by a 'Unix-from' header.  The
        header block is terminated either by the end of the input or by a
        blank line.

        _class is the class to instantiate for new message objects when they
        must be created.  This class must have a constructor that can take
        zero arguments.  Default is Message.Message.
        """

    @overload
    def __init__(self, _class: None = None, *, policy: Policy[_MessageT]) -> None: ...
    @overload
    def __init__(self, _class: Callable[[], _MessageT], *, policy: Policy[_MessageT] = ...) -> None: ...
    def parse(self, fp: _WrappedBuffer, headersonly: bool = False) -> _MessageT:
        """Create a message structure from the data in a binary file.

        Reads all the data from the file and returns the root of the message
        structure.  Optional headersonly is a flag specifying whether to stop
        parsing after reading the headers or not.  The default is False,
        meaning it parses the entire contents of the file.
        """

    def parsebytes(self, text: bytes | bytearray, headersonly: bool = False) -> _MessageT:
        """Create a message structure from a byte string.

        Returns the root of the message structure.  Optional headersonly is a
        flag specifying whether to stop parsing after reading the headers or
        not.  The default is False, meaning it parses the entire contents of
        the file.
        """

class BytesHeaderParser(BytesParser[_MessageT]):
    def parse(self, fp: _WrappedBuffer, headersonly: bool = True) -> _MessageT: ...
    def parsebytes(self, text: bytes | bytearray, headersonly: bool = True) -> _MessageT: ...



"""Generic output formatting.

Formatter objects transform an abstract flow of formatting events into
specific output events on writer objects. Formatters manage several stack
structures to allow various properties of a writer object to be changed and
restored; writers need not be able to handle relative changes nor any sort
of ``change back'' operation. Specific writer properties which may be
controlled via formatter objects are horizontal alignment, font, and left
margin indentations. A mechanism is provided which supports providing
arbitrary, non-exclusive style settings to a writer as well. Additional
interfaces facilitate formatting events which are not reversible, such as
paragraph separation.

Writer objects encapsulate device interfaces. Abstract devices, such as
file formats, are supported as well as physical devices. The provided
implementations all work with abstract devices. The interface makes
available mechanisms for setting the properties which formatter objects
manage and inserting data into the output.
"""

from collections.abc import Iterable
from typing import IO, Any
from typing_extensions import TypeAlias

AS_IS: None
_FontType: TypeAlias = tuple[str, bool, bool, bool]
_StylesType: TypeAlias = tuple[Any, ...]

class NullFormatter:
    """A formatter which does nothing.

    If the writer parameter is omitted, a NullWriter instance is created.
    No methods of the writer are called by NullFormatter instances.

    Implementations should inherit from this class if implementing a writer
    interface but don't need to inherit any implementation.

    """

    writer: NullWriter | None
    def __init__(self, writer: NullWriter | None = None) -> None: ...
    def end_paragraph(self, blankline: int) -> None: ...
    def add_line_break(self) -> None: ...
    def add_hor_rule(self, *args: Any, **kw: Any) -> None: ...
    def add_label_data(self, format: str, counter: int, blankline: int | None = None) -> None: ...
    def add_flowing_data(self, data: str) -> None: ...
    def add_literal_data(self, data: str) -> None: ...
    def flush_softspace(self) -> None: ...
    def push_alignment(self, align: str | None) -> None: ...
    def pop_alignment(self) -> None: ...
    def push_font(self, x: _FontType) -> None: ...
    def pop_font(self) -> None: ...
    def push_margin(self, margin: int) -> None: ...
    def pop_margin(self) -> None: ...
    def set_spacing(self, spacing: str | None) -> None: ...
    def push_style(self, *styles: _StylesType) -> None: ...
    def pop_style(self, n: int = 1) -> None: ...
    def assert_line_data(self, flag: int = 1) -> None: ...

class AbstractFormatter:
    """The standard formatter.

    This implementation has demonstrated wide applicability to many writers,
    and may be used directly in most circumstances.  It has been used to
    implement a full-featured World Wide Web browser.

    """

    writer: NullWriter
    align: str | None
    align_stack: list[str | None]
    font_stack: list[_FontType]
    margin_stack: list[int]
    spacing: str | None
    style_stack: Any
    nospace: int
    softspace: int
    para_end: int
    parskip: int
    hard_break: int
    have_label: int
    def __init__(self, writer: NullWriter) -> None: ...
    def end_paragraph(self, blankline: int) -> None: ...
    def add_line_break(self) -> None: ...
    def add_hor_rule(self, *args: Any, **kw: Any) -> None: ...
    def add_label_data(self, format: str, counter: int, blankline: int | None = None) -> None: ...
    def format_counter(self, format: Iterable[str], counter: int) -> str: ...
    def format_letter(self, case: str, counter: int) -> str: ...
    def format_roman(self, case: str, counter: int) -> str: ...
    def add_flowing_data(self, data: str) -> None: ...
    def add_literal_data(self, data: str) -> None: ...
    def flush_softspace(self) -> None: ...
    def push_alignment(self, align: str | None) -> None: ...
    def pop_alignment(self) -> None: ...
    def push_font(self, font: _FontType) -> None: ...
    def pop_font(self) -> None: ...
    def push_margin(self, margin: int) -> None: ...
    def pop_margin(self) -> None: ...
    def set_spacing(self, spacing: str | None) -> None: ...
    def push_style(self, *styles: _StylesType) -> None: ...
    def pop_style(self, n: int = 1) -> None: ...
    def assert_line_data(self, flag: int = 1) -> None: ...

class NullWriter:
    """Minimal writer interface to use in testing & inheritance.

    A writer which only provides the interface definition; no actions are
    taken on any methods.  This should be the base class for all writers
    which do not need to inherit any implementation methods.

    """

    def flush(self) -> None: ...
    def new_alignment(self, align: str | None) -> None: ...
    def new_font(self, font: _FontType) -> None: ...
    def new_margin(self, margin: int, level: int) -> None: ...
    def new_spacing(self, spacing: str | None) -> None: ...
    def new_styles(self, styles: tuple[Any, ...]) -> None: ...
    def send_paragraph(self, blankline: int) -> None: ...
    def send_line_break(self) -> None: ...
    def send_hor_rule(self, *args: Any, **kw: Any) -> None: ...
    def send_label_data(self, data: str) -> None: ...
    def send_flowing_data(self, data: str) -> None: ...
    def send_literal_data(self, data: str) -> None: ...

class AbstractWriter(NullWriter):
    """A writer which can be used in debugging formatters, but not much else.

    Each method simply announces itself by printing its name and
    arguments on standard output.

    """

class DumbWriter(NullWriter):
    """Simple writer class which writes output on the file object passed in
    as the file parameter or, if file is omitted, on standard output.  The
    output is simply word-wrapped to the number of columns specified by
    the maxcol parameter.  This class is suitable for reflowing a sequence
    of paragraphs.

    """

    file: IO[str]
    maxcol: int
    def __init__(self, file: IO[str] | None = None, maxcol: int = 72) -> None: ...
    def reset(self) -> None: ...

def test(file: str | None = None) -> None: ...



"""Base classes for server/gateway implementations"""

from _typeshed import OptExcInfo
from _typeshed.wsgi import ErrorStream, InputStream, StartResponse, WSGIApplication, WSGIEnvironment
from abc import abstractmethod
from collections.abc import Callable, MutableMapping
from typing import IO

from .headers import Headers
from .util import FileWrapper

__all__ = ["BaseHandler", "SimpleHandler", "BaseCGIHandler", "CGIHandler", "IISCGIHandler", "read_environ"]

def format_date_time(timestamp: float | None) -> str: ...  # undocumented
def read_environ() -> dict[str, str]:
    """Read environment, fixing HTTP variables"""

class BaseHandler:
    """Manage the invocation of a WSGI application"""

    wsgi_version: tuple[int, int]  # undocumented
    wsgi_multithread: bool
    wsgi_multiprocess: bool
    wsgi_run_once: bool

    origin_server: bool
    http_version: str
    server_software: str | None

    os_environ: MutableMapping[str, str]

    wsgi_file_wrapper: type[FileWrapper] | None
    headers_class: type[Headers]  # undocumented

    traceback_limit: int | None
    error_status: str
    error_headers: list[tuple[str, str]]
    error_body: bytes
    def run(self, application: WSGIApplication) -> None:
        """Invoke the application"""

    def setup_environ(self) -> None:
        """Set up the environment for one request"""

    def finish_response(self) -> None:
        """Send any iterable data, then close self and the iterable

        Subclasses intended for use in asynchronous servers will
        want to redefine this method, such that it sets up callbacks
        in the event loop to iterate over the data, and to call
        'self.close()' once the response is finished.
        """

    def get_scheme(self) -> str:
        """Return the URL scheme being used"""

    def set_content_length(self) -> None:
        """Compute Content-Length or switch to chunked encoding if possible"""

    def cleanup_headers(self) -> None:
        """Make any necessary header changes or defaults

        Subclasses can extend this to add other defaults.
        """

    def start_response(
        self, status: str, headers: list[tuple[str, str]], exc_info: OptExcInfo | None = None
    ) -> Callable[[bytes], None]:
        """'start_response()' callable as specified by PEP 3333"""

    def send_preamble(self) -> None:
        """Transmit version/status/date/server, via self._write()"""

    def write(self, data: bytes) -> None:
        """'write()' callable as specified by PEP 3333"""

    def sendfile(self) -> bool:
        """Platform-specific file transmission

        Override this method in subclasses to support platform-specific
        file transmission.  It is only called if the application's
        return iterable ('self.result') is an instance of
        'self.wsgi_file_wrapper'.

        This method should return a true value if it was able to actually
        transmit the wrapped file-like object using a platform-specific
        approach.  It should return a false value if normal iteration
        should be used instead.  An exception can be raised to indicate
        that transmission was attempted, but failed.

        NOTE: this method should call 'self.send_headers()' if
        'self.headers_sent' is false and it is going to attempt direct
        transmission of the file.
        """

    def finish_content(self) -> None:
        """Ensure headers and content have both been sent"""

    def close(self) -> None:
        """Close the iterable (if needed) and reset all instance vars

        Subclasses may want to also drop the client connection.
        """

    def send_headers(self) -> None:
        """Transmit headers to the client, via self._write()"""

    def result_is_file(self) -> bool:
        """True if 'self.result' is an instance of 'self.wsgi_file_wrapper'"""

    def client_is_modern(self) -> bool:
        """True if client can accept status and headers"""

    def log_exception(self, exc_info: OptExcInfo) -> None:
        """Log the 'exc_info' tuple in the server log

        Subclasses may override to retarget the output or change its format.
        """

    def handle_error(self) -> None:
        """Log current error, and send error output to client if possible"""

    def error_output(self, environ: WSGIEnvironment, start_response: StartResponse) -> list[bytes]:
        """WSGI mini-app to create error output

        By default, this just uses the 'error_status', 'error_headers',
        and 'error_body' attributes to generate an output page.  It can
        be overridden in a subclass to dynamically generate diagnostics,
        choose an appropriate message for the user's preferred language, etc.

        Note, however, that it's not recommended from a security perspective to
        spit out diagnostics to any old user; ideally, you should have to do
        something special to enable diagnostic output, which is why we don't
        include any here!
        """

    @abstractmethod
    def _write(self, data: bytes) -> None:
        """Override in subclass to buffer data for send to client

        It's okay if this method actually transmits the data; BaseHandler
        just separates write and flush operations for greater efficiency
        when the underlying system actually has such a distinction.
        """

    @abstractmethod
    def _flush(self) -> None:
        """Override in subclass to force sending of recent '_write()' calls

        It's okay if this method is a no-op (i.e., if '_write()' actually
        sends the data.
        """

    @abstractmethod
    def get_stdin(self) -> InputStream:
        """Override in subclass to return suitable 'wsgi.input'"""

    @abstractmethod
    def get_stderr(self) -> ErrorStream:
        """Override in subclass to return suitable 'wsgi.errors'"""

    @abstractmethod
    def add_cgi_vars(self) -> None:
        """Override in subclass to insert CGI variables in 'self.environ'"""

class SimpleHandler(BaseHandler):
    """Handler that's just initialized with streams, environment, etc.

    This handler subclass is intended for synchronous HTTP/1.0 origin servers,
    and handles sending the entire response output, given the correct inputs.

    Usage::

        handler = SimpleHandler(
            inp,out,err,env, multithread=False, multiprocess=True
        )
        handler.run(app)
    """

    stdin: InputStream
    stdout: IO[bytes]
    stderr: ErrorStream
    base_env: MutableMapping[str, str]
    def __init__(
        self,
        stdin: InputStream,
        stdout: IO[bytes],
        stderr: ErrorStream,
        environ: MutableMapping[str, str],
        multithread: bool = True,
        multiprocess: bool = False,
    ) -> None: ...
    def get_stdin(self) -> InputStream: ...
    def get_stderr(self) -> ErrorStream: ...
    def add_cgi_vars(self) -> None: ...
    def _write(self, data: bytes) -> None: ...
    def _flush(self) -> None: ...

class BaseCGIHandler(SimpleHandler):
    """CGI-like systems using input/output/error streams and environ mapping

    Usage::

        handler = BaseCGIHandler(inp,out,err,env)
        handler.run(app)

    This handler class is useful for gateway protocols like ReadyExec and
    FastCGI, that have usable input/output/error streams and an environment
    mapping.  It's also the base class for CGIHandler, which just uses
    sys.stdin, os.environ, and so on.

    The constructor also takes keyword arguments 'multithread' and
    'multiprocess' (defaulting to 'True' and 'False' respectively) to control
    the configuration sent to the application.  It sets 'origin_server' to
    False (to enable CGI-like output), and assumes that 'wsgi.run_once' is
    False.
    """

class CGIHandler(BaseCGIHandler):
    """CGI-based invocation via sys.stdin/stdout/stderr and os.environ

    Usage::

        CGIHandler().run(app)

    The difference between this class and BaseCGIHandler is that it always
    uses 'wsgi.run_once' of 'True', 'wsgi.multithread' of 'False', and
    'wsgi.multiprocess' of 'True'.  It does not take any initialization
    parameters, but always uses 'sys.stdin', 'os.environ', and friends.

    If you need to override any of these parameters, use BaseCGIHandler
    instead.
    """

    def __init__(self) -> None: ...

class IISCGIHandler(BaseCGIHandler):
    """CGI-based invocation with workaround for IIS path bug

    This handler should be used in preference to CGIHandler when deploying on
    Microsoft IIS without having set the config allowPathInfo option (IIS>=7)
    or metabase allowPathInfoForScriptMappings (IIS<7).
    """

    def __init__(self) -> None: ...



"""Decimal fixed-point and floating-point arithmetic.

This is an implementation of decimal floating-point arithmetic based on
the General Decimal Arithmetic Specification:

    http://speleotrove.com/decimal/decarith.html

and IEEE standard 854-1987:

    http://en.wikipedia.org/wiki/IEEE_854-1987

Decimal floating point has finite precision with arbitrarily large bounds.

The purpose of this module is to support arithmetic using familiar
"schoolhouse" rules and to avoid some of the tricky representation
issues associated with binary floating point.  The package is especially
useful for financial applications or for contexts where users have
expectations that are at odds with binary floating point (for instance,
in binary floating point, 1.00 % 0.1 gives 0.09999999999999995 instead
of 0.0; Decimal('1.00') % Decimal('0.1') returns the expected
Decimal('0.00')).

Here are some examples of using the decimal module:

>>> from decimal import *
>>> setcontext(ExtendedContext)
>>> Decimal(0)
Decimal('0')
>>> Decimal('1')
Decimal('1')
>>> Decimal('-.0123')
Decimal('-0.0123')
>>> Decimal(123456)
Decimal('123456')
>>> Decimal('123.45e12345678')
Decimal('1.2345E+12345680')
>>> Decimal('1.33') + Decimal('1.27')
Decimal('2.60')
>>> Decimal('12.34') + Decimal('3.87') - Decimal('18.41')
Decimal('-2.20')
>>> dig = Decimal(1)
>>> print(dig / Decimal(3))
0.333333333
>>> getcontext().prec = 18
>>> print(dig / Decimal(3))
0.333333333333333333
>>> print(dig.sqrt())
1
>>> print(Decimal(3).sqrt())
1.73205080756887729
>>> print(Decimal(3) ** 123)
4.85192780976896427E+58
>>> inf = Decimal(1) / Decimal(0)
>>> print(inf)
Infinity
>>> neginf = Decimal(-1) / Decimal(0)
>>> print(neginf)
-Infinity
>>> print(neginf + inf)
NaN
>>> print(neginf * inf)
-Infinity
>>> print(dig / 0)
Infinity
>>> getcontext().traps[DivisionByZero] = 1
>>> print(dig / 0)
Traceback (most recent call last):
  ...
  ...
  ...
decimal.DivisionByZero: x / 0
>>> c = Context()
>>> c.traps[InvalidOperation] = 0
>>> print(c.flags[InvalidOperation])
0
>>> c.divide(Decimal(0), Decimal(0))
Decimal('NaN')
>>> c.traps[InvalidOperation] = 1
>>> print(c.flags[InvalidOperation])
1
>>> c.flags[InvalidOperation] = 0
>>> print(c.flags[InvalidOperation])
0
>>> print(c.divide(Decimal(0), Decimal(0)))
Traceback (most recent call last):
  ...
  ...
  ...
decimal.InvalidOperation: 0 / 0
>>> print(c.flags[InvalidOperation])
1
>>> c.flags[InvalidOperation] = 0
>>> c.traps[InvalidOperation] = 0
>>> print(c.divide(Decimal(0), Decimal(0)))
NaN
>>> print(c.flags[InvalidOperation])
1
>>>
"""

import numbers
import sys
from _decimal import (
    HAVE_CONTEXTVAR as HAVE_CONTEXTVAR,
    HAVE_THREADS as HAVE_THREADS,
    MAX_EMAX as MAX_EMAX,
    MAX_PREC as MAX_PREC,
    MIN_EMIN as MIN_EMIN,
    MIN_ETINY as MIN_ETINY,
    ROUND_05UP as ROUND_05UP,
    ROUND_CEILING as ROUND_CEILING,
    ROUND_DOWN as ROUND_DOWN,
    ROUND_FLOOR as ROUND_FLOOR,
    ROUND_HALF_DOWN as ROUND_HALF_DOWN,
    ROUND_HALF_EVEN as ROUND_HALF_EVEN,
    ROUND_HALF_UP as ROUND_HALF_UP,
    ROUND_UP as ROUND_UP,
    BasicContext as BasicContext,
    DefaultContext as DefaultContext,
    ExtendedContext as ExtendedContext,
    __libmpdec_version__ as __libmpdec_version__,
    __version__ as __version__,
    getcontext as getcontext,
    localcontext as localcontext,
    setcontext as setcontext,
)
from collections.abc import Container, Sequence
from types import TracebackType
from typing import Any, ClassVar, Literal, NamedTuple, final, overload, type_check_only
from typing_extensions import Self, TypeAlias, disjoint_base

if sys.version_info >= (3, 14):
    from _decimal import IEEE_CONTEXT_MAX_BITS as IEEE_CONTEXT_MAX_BITS, IEEEContext as IEEEContext

_Decimal: TypeAlias = Decimal | int
_DecimalNew: TypeAlias = Decimal | float | str | tuple[int, Sequence[int], int]
_ComparableNum: TypeAlias = Decimal | float | numbers.Rational
_TrapType: TypeAlias = type[DecimalException]

# At runtime, these classes are implemented in C as part of "_decimal".
# However, they consider themselves to live in "decimal", so we'll put them here.

# This type isn't exposed at runtime. It calls itself decimal.ContextManager
@final
@type_check_only
class _ContextManager:
    def __init__(self, new_context: Context) -> None: ...
    def __enter__(self) -> Context: ...
    def __exit__(self, t: type[BaseException] | None, v: BaseException | None, tb: TracebackType | None) -> None: ...

class DecimalTuple(NamedTuple):
    """DecimalTuple(sign, digits, exponent)"""

    sign: int
    digits: tuple[int, ...]
    exponent: int | Literal["n", "N", "F"]

class DecimalException(ArithmeticError): ...
class Clamped(DecimalException): ...
class InvalidOperation(DecimalException): ...
class ConversionSyntax(InvalidOperation): ...
class DivisionByZero(DecimalException, ZeroDivisionError): ...
class DivisionImpossible(InvalidOperation): ...
class DivisionUndefined(InvalidOperation, ZeroDivisionError): ...
class Inexact(DecimalException): ...
class InvalidContext(InvalidOperation): ...
class Rounded(DecimalException): ...
class Subnormal(DecimalException): ...
class Overflow(Inexact, Rounded): ...
class Underflow(Inexact, Rounded, Subnormal): ...
class FloatOperation(DecimalException, TypeError): ...

@disjoint_base
class Decimal:
    """Construct a new Decimal object. 'value' can be an integer, string, tuple,
    or another Decimal object. If no value is given, return Decimal('0'). The
    context does not affect the conversion and is only passed to determine if
    the InvalidOperation trap is active.

    """

    def __new__(cls, value: _DecimalNew = "0", context: Context | None = None) -> Self: ...
    if sys.version_info >= (3, 14):
        @classmethod
        def from_number(cls, number: Decimal | float, /) -> Self:
            """Class method that converts a real number to a decimal number, exactly.

            >>> Decimal.from_number(314)              # int
            Decimal('314')
            >>> Decimal.from_number(0.1)              # float
            Decimal('0.1000000000000000055511151231257827021181583404541015625')
            >>> Decimal.from_number(Decimal('3.14'))  # another decimal instance
            Decimal('3.14')


            """

    @classmethod
    def from_float(cls, f: float, /) -> Self:
        """Class method that converts a float to a decimal number, exactly.
        Since 0.1 is not exactly representable in binary floating point,
        Decimal.from_float(0.1) is not the same as Decimal('0.1').

            >>> Decimal.from_float(0.1)
            Decimal('0.1000000000000000055511151231257827021181583404541015625')
            >>> Decimal.from_float(float('nan'))
            Decimal('NaN')
            >>> Decimal.from_float(float('inf'))
            Decimal('Infinity')
            >>> Decimal.from_float(float('-inf'))
            Decimal('-Infinity')


        """

    def __bool__(self) -> bool:
        """True if self else False"""

    def compare(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Compare self to other.  Return a decimal value:

        a or b is a NaN ==> Decimal('NaN')
        a < b           ==> Decimal('-1')
        a == b          ==> Decimal('0')
        a > b           ==> Decimal('1')

        """

    def __hash__(self) -> int: ...
    def as_tuple(self) -> DecimalTuple:
        """Return a tuple representation of the number."""

    def as_integer_ratio(self) -> tuple[int, int]:
        """Decimal.as_integer_ratio() -> (int, int)

        Return a pair of integers, whose ratio is exactly equal to the original
        Decimal and with a positive denominator. The ratio is in lowest terms.
        Raise OverflowError on infinities and a ValueError on NaNs.

        """

    def to_eng_string(self, context: Context | None = None) -> str:
        """Convert to an engineering-type string.  Engineering notation has an exponent
        which is a multiple of 3, so there are up to 3 digits left of the decimal
        place. For example, Decimal('123E+1') is converted to Decimal('1.23E+3').

        The value of context.capitals determines whether the exponent sign is lower
        or upper case. Otherwise, the context does not affect the operation.

        """

    def __abs__(self) -> Decimal:
        """abs(self)"""

    def __add__(self, value: _Decimal, /) -> Decimal:
        """Return self+value."""

    def __divmod__(self, value: _Decimal, /) -> tuple[Decimal, Decimal]:
        """Return divmod(self, value)."""

    def __eq__(self, value: object, /) -> bool: ...
    def __floordiv__(self, value: _Decimal, /) -> Decimal:
        """Return self//value."""

    def __ge__(self, value: _ComparableNum, /) -> bool: ...
    def __gt__(self, value: _ComparableNum, /) -> bool: ...
    def __le__(self, value: _ComparableNum, /) -> bool: ...
    def __lt__(self, value: _ComparableNum, /) -> bool: ...
    def __mod__(self, value: _Decimal, /) -> Decimal:
        """Return self%value."""

    def __mul__(self, value: _Decimal, /) -> Decimal:
        """Return self*value."""

    def __neg__(self) -> Decimal:
        """-self"""

    def __pos__(self) -> Decimal:
        """+self"""

    def __pow__(self, value: _Decimal, mod: _Decimal | None = None, /) -> Decimal:
        """Return pow(self, value, mod)."""

    def __radd__(self, value: _Decimal, /) -> Decimal:
        """Return value+self."""

    def __rdivmod__(self, value: _Decimal, /) -> tuple[Decimal, Decimal]:
        """Return divmod(value, self)."""

    def __rfloordiv__(self, value: _Decimal, /) -> Decimal:
        """Return value//self."""

    def __rmod__(self, value: _Decimal, /) -> Decimal:
        """Return value%self."""

    def __rmul__(self, value: _Decimal, /) -> Decimal:
        """Return value*self."""

    def __rsub__(self, value: _Decimal, /) -> Decimal:
        """Return value-self."""

    def __rtruediv__(self, value: _Decimal, /) -> Decimal:
        """Return value/self."""

    def __sub__(self, value: _Decimal, /) -> Decimal:
        """Return self-value."""

    def __truediv__(self, value: _Decimal, /) -> Decimal:
        """Return self/value."""

    def remainder_near(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Return the remainder from dividing self by other.  This differs from
        self % other in that the sign of the remainder is chosen so as to minimize
        its absolute value. More precisely, the return value is self - n * other
        where n is the integer nearest to the exact value of self / other, and
        if two integers are equally near then the even one is chosen.

        If the result is zero then its sign will be the sign of self.

        """

    def __float__(self) -> float:
        """float(self)"""

    def __int__(self) -> int:
        """int(self)"""

    def __trunc__(self) -> int: ...
    @property
    def real(self) -> Decimal: ...
    @property
    def imag(self) -> Decimal: ...
    def conjugate(self) -> Decimal:
        """Return self."""

    def __complex__(self) -> complex: ...
    @overload
    def __round__(self) -> int: ...
    @overload
    def __round__(self, ndigits: int, /) -> Decimal: ...
    def __floor__(self) -> int: ...
    def __ceil__(self) -> int: ...
    def fma(self, other: _Decimal, third: _Decimal, context: Context | None = None) -> Decimal:
        """Fused multiply-add.  Return self*other+third with no rounding of the
        intermediate product self*other.

            >>> Decimal(2).fma(3, 5)
            Decimal('11')


        """

    def __rpow__(self, value: _Decimal, mod: Context | None = None, /) -> Decimal:
        """Return pow(value, self, mod)."""

    def normalize(self, context: Context | None = None) -> Decimal:
        """Normalize the number by stripping the rightmost trailing zeros and
        converting any result equal to Decimal('0') to Decimal('0e0').  Used
        for producing canonical values for members of an equivalence class.
        For example, Decimal('32.100') and Decimal('0.321000e+2') both normalize
        to the equivalent value Decimal('32.1').

        """

    def quantize(self, exp: _Decimal, rounding: str | None = None, context: Context | None = None) -> Decimal:
        """Return a value equal to the first operand after rounding and having the
        exponent of the second operand.

            >>> Decimal('1.41421356').quantize(Decimal('1.000'))
            Decimal('1.414')

        Unlike other operations, if the length of the coefficient after the quantize
        operation would be greater than precision, then an InvalidOperation is signaled.
        This guarantees that, unless there is an error condition, the quantized exponent
        is always equal to that of the right-hand operand.

        Also unlike other operations, quantize never signals Underflow, even if the
        result is subnormal and inexact.

        If the exponent of the second operand is larger than that of the first, then
        rounding may be necessary. In this case, the rounding mode is determined by the
        rounding argument if given, else by the given context argument; if neither
        argument is given, the rounding mode of the current thread's context is used.

        """

    def same_quantum(self, other: _Decimal, context: Context | None = None) -> bool:
        """Test whether self and other have the same exponent or whether both are NaN.

        This operation is unaffected by context and is quiet: no flags are changed
        and no rounding is performed. As an exception, the C version may raise
        InvalidOperation if the second operand cannot be converted exactly.

        """

    def to_integral_exact(self, rounding: str | None = None, context: Context | None = None) -> Decimal:
        """Round to the nearest integer, signaling Inexact or Rounded as appropriate if
        rounding occurs.  The rounding mode is determined by the rounding parameter
        if given, else by the given context. If neither parameter is given, then the
        rounding mode of the current default context is used.

        """

    def to_integral_value(self, rounding: str | None = None, context: Context | None = None) -> Decimal:
        """Round to the nearest integer without signaling Inexact or Rounded.  The
        rounding mode is determined by the rounding parameter if given, else by
        the given context. If neither parameter is given, then the rounding mode
        of the current default context is used.

        """

    def to_integral(self, rounding: str | None = None, context: Context | None = None) -> Decimal:
        """Identical to the to_integral_value() method.  The to_integral() name has been
        kept for compatibility with older versions.

        """

    def sqrt(self, context: Context | None = None) -> Decimal:
        """Return the square root of the argument to full precision. The result is
        correctly rounded using the ROUND_HALF_EVEN rounding mode.

        """

    def max(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Maximum of self and other.  If one operand is a quiet NaN and the other is
        numeric, the numeric operand is returned.

        """

    def min(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Minimum of self and other. If one operand is a quiet NaN and the other is
        numeric, the numeric operand is returned.

        """

    def adjusted(self) -> int:
        """Return the adjusted exponent of the number.  Defined as exp + digits - 1."""

    def canonical(self) -> Decimal:
        """Return the canonical encoding of the argument.  Currently, the encoding
        of a Decimal instance is always canonical, so this operation returns its
        argument unchanged.

        """

    def compare_signal(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Identical to compare, except that all NaNs signal."""

    def compare_total(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Compare two operands using their abstract representation rather than
        their numerical value.  Similar to the compare() method, but the result
        gives a total ordering on Decimal instances.  Two Decimal instances with
        the same numeric value but different representations compare unequal
        in this ordering:

            >>> Decimal('12.0').compare_total(Decimal('12'))
            Decimal('-1')

        Quiet and signaling NaNs are also included in the total ordering. The result
        of this function is Decimal('0') if both operands have the same representation,
        Decimal('-1') if the first operand is lower in the total order than the second,
        and Decimal('1') if the first operand is higher in the total order than the
        second operand. See the specification for details of the total order.

        This operation is unaffected by context and is quiet: no flags are changed
        and no rounding is performed. As an exception, the C version may raise
        InvalidOperation if the second operand cannot be converted exactly.

        """

    def compare_total_mag(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Compare two operands using their abstract representation rather than their
        value as in compare_total(), but ignoring the sign of each operand.

        x.compare_total_mag(y) is equivalent to x.copy_abs().compare_total(y.copy_abs()).

        This operation is unaffected by context and is quiet: no flags are changed
        and no rounding is performed. As an exception, the C version may raise
        InvalidOperation if the second operand cannot be converted exactly.

        """

    def copy_abs(self) -> Decimal:
        """Return the absolute value of the argument.  This operation is unaffected by
        context and is quiet: no flags are changed and no rounding is performed.

        """

    def copy_negate(self) -> Decimal:
        """Return the negation of the argument.  This operation is unaffected by context
        and is quiet: no flags are changed and no rounding is performed.

        """

    def copy_sign(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Return a copy of the first operand with the sign set to be the same as the
        sign of the second operand. For example:

            >>> Decimal('2.3').copy_sign(Decimal('-1.5'))
            Decimal('-2.3')

        This operation is unaffected by context and is quiet: no flags are changed
        and no rounding is performed. As an exception, the C version may raise
        InvalidOperation if the second operand cannot be converted exactly.

        """

    def exp(self, context: Context | None = None) -> Decimal:
        """Return the value of the (natural) exponential function e**x at the given
        number.  The function always uses the ROUND_HALF_EVEN mode and the result
        is correctly rounded.

        """

    def is_canonical(self) -> bool:
        """Return True if the argument is canonical and False otherwise.  Currently,
        a Decimal instance is always canonical, so this operation always returns
        True.

        """

    def is_finite(self) -> bool:
        """Return True if the argument is a finite number, and False if the argument
        is infinite or a NaN.

        """

    def is_infinite(self) -> bool:
        """Return True if the argument is either positive or negative infinity and
        False otherwise.

        """

    def is_nan(self) -> bool:
        """Return True if the argument is a (quiet or signaling) NaN and False
        otherwise.

        """

    def is_normal(self, context: Context | None = None) -> bool:
        """Return True if the argument is a normal finite non-zero number with an
        adjusted exponent greater than or equal to Emin. Return False if the
        argument is zero, subnormal, infinite or a NaN.

        """

    def is_qnan(self) -> bool:
        """Return True if the argument is a quiet NaN, and False otherwise."""

    def is_signed(self) -> bool:
        """Return True if the argument has a negative sign and False otherwise.
        Note that both zeros and NaNs can carry signs.

        """

    def is_snan(self) -> bool:
        """Return True if the argument is a signaling NaN and False otherwise."""

    def is_subnormal(self, context: Context | None = None) -> bool:
        """Return True if the argument is subnormal, and False otherwise. A number is
        subnormal if it is non-zero, finite, and has an adjusted exponent less
        than Emin.

        """

    def is_zero(self) -> bool:
        """Return True if the argument is a (positive or negative) zero and False
        otherwise.

        """

    def ln(self, context: Context | None = None) -> Decimal:
        """Return the natural (base e) logarithm of the operand. The function always
        uses the ROUND_HALF_EVEN mode and the result is correctly rounded.

        """

    def log10(self, context: Context | None = None) -> Decimal:
        """Return the base ten logarithm of the operand. The function always uses the
        ROUND_HALF_EVEN mode and the result is correctly rounded.

        """

    def logb(self, context: Context | None = None) -> Decimal:
        """For a non-zero number, return the adjusted exponent of the operand as a
        Decimal instance.  If the operand is a zero, then Decimal('-Infinity') is
        returned and the DivisionByZero condition is raised. If the operand is
        an infinity then Decimal('Infinity') is returned.

        """

    def logical_and(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Return the digit-wise 'and' of the two (logical) operands."""

    def logical_invert(self, context: Context | None = None) -> Decimal:
        """Return the digit-wise inversion of the (logical) operand."""

    def logical_or(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Return the digit-wise 'or' of the two (logical) operands."""

    def logical_xor(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Return the digit-wise 'exclusive or' of the two (logical) operands."""

    def max_mag(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Similar to the max() method, but the comparison is done using the absolute
        values of the operands.

        """

    def min_mag(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Similar to the min() method, but the comparison is done using the absolute
        values of the operands.

        """

    def next_minus(self, context: Context | None = None) -> Decimal:
        """Return the largest number representable in the given context (or in the
        current default context if no context is given) that is smaller than the
        given operand.

        """

    def next_plus(self, context: Context | None = None) -> Decimal:
        """Return the smallest number representable in the given context (or in the
        current default context if no context is given) that is larger than the
        given operand.

        """

    def next_toward(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """If the two operands are unequal, return the number closest to the first
        operand in the direction of the second operand.  If both operands are
        numerically equal, return a copy of the first operand with the sign set
        to be the same as the sign of the second operand.

        """

    def number_class(self, context: Context | None = None) -> str:
        """Return a string describing the class of the operand.  The returned value
        is one of the following ten strings:

            * '-Infinity', indicating that the operand is negative infinity.
            * '-Normal', indicating that the operand is a negative normal number.
            * '-Subnormal', indicating that the operand is negative and subnormal.
            * '-Zero', indicating that the operand is a negative zero.
            * '+Zero', indicating that the operand is a positive zero.
            * '+Subnormal', indicating that the operand is positive and subnormal.
            * '+Normal', indicating that the operand is a positive normal number.
            * '+Infinity', indicating that the operand is positive infinity.
            * 'NaN', indicating that the operand is a quiet NaN (Not a Number).
            * 'sNaN', indicating that the operand is a signaling NaN.


        """

    def radix(self) -> Decimal:
        """Return Decimal(10), the radix (base) in which the Decimal class does
        all its arithmetic. Included for compatibility with the specification.

        """

    def rotate(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Return the result of rotating the digits of the first operand by an amount
        specified by the second operand.  The second operand must be an integer in
        the range -precision through precision. The absolute value of the second
        operand gives the number of places to rotate. If the second operand is
        positive then rotation is to the left; otherwise rotation is to the right.
        The coefficient of the first operand is padded on the left with zeros to
        length precision if necessary. The sign and exponent of the first operand are
        unchanged.

        """

    def scaleb(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Return the first operand with the exponent adjusted the second.  Equivalently,
        return the first operand multiplied by 10**other. The second operand must be
        an integer.

        """

    def shift(self, other: _Decimal, context: Context | None = None) -> Decimal:
        """Return the result of shifting the digits of the first operand by an amount
        specified by the second operand.  The second operand must be an integer in
        the range -precision through precision. The absolute value of the second
        operand gives the number of places to shift. If the second operand is
        positive, then the shift is to the left; otherwise the shift is to the
        right. Digits shifted into the coefficient are zeros. The sign and exponent
        of the first operand are unchanged.

        """

    def __reduce__(self) -> tuple[type[Self], tuple[str]]: ...
    def __copy__(self) -> Self: ...
    def __deepcopy__(self, memo: Any, /) -> Self: ...
    def __format__(self, specifier: str, context: Context | None = None, /) -> str: ...

@disjoint_base
class Context:
    """The context affects almost all operations and controls rounding,
    Over/Underflow, raising of exceptions and much more.  A new context
    can be constructed as follows:

        >>> c = Context(prec=28, Emin=-425000000, Emax=425000000,
        ...             rounding=ROUND_HALF_EVEN, capitals=1, clamp=1,
        ...             traps=[InvalidOperation, DivisionByZero, Overflow],
        ...             flags=[])
        >>>


    """

    # TODO: Context doesn't allow you to delete *any* attributes from instances of the class at runtime,
    # even settable attributes like `prec` and `rounding`,
    # but that's inexpressible in the stub.
    # Type checkers either ignore it or misinterpret it
    # if you add a `def __delattr__(self, name: str, /) -> NoReturn` method to the stub
    prec: int
    rounding: str
    Emin: int
    Emax: int
    capitals: int
    clamp: int
    traps: dict[_TrapType, bool]
    flags: dict[_TrapType, bool]
    def __init__(
        self,
        prec: int | None = None,
        rounding: str | None = None,
        Emin: int | None = None,
        Emax: int | None = None,
        capitals: int | None = None,
        clamp: int | None = None,
        flags: dict[_TrapType, bool] | Container[_TrapType] | None = None,
        traps: dict[_TrapType, bool] | Container[_TrapType] | None = None,
    ) -> None: ...
    def __reduce__(self) -> tuple[type[Self], tuple[Any, ...]]: ...
    def clear_flags(self) -> None:
        """Reset all flags to False."""

    def clear_traps(self) -> None:
        """Set all traps to False."""

    def copy(self) -> Context:
        """Return a duplicate of the context with all flags cleared."""

    def __copy__(self) -> Context: ...
    # see https://github.com/python/cpython/issues/94107
    __hash__: ClassVar[None]  # type: ignore[assignment]
    def Etiny(self) -> int:
        """Return a value equal to Emin - prec + 1, which is the minimum exponent value
        for subnormal results.  When underflow occurs, the exponent is set to Etiny.

        """

    def Etop(self) -> int:
        """Return a value equal to Emax - prec + 1.  This is the maximum exponent
        if the _clamp field of the context is set to 1 (IEEE clamp mode).  Etop()
        must not be negative.

        """

    def create_decimal(self, num: _DecimalNew = "0", /) -> Decimal:
        """Create a new Decimal instance from num, using self as the context. Unlike the
        Decimal constructor, this function observes the context limits.

        """

    def create_decimal_from_float(self, f: float, /) -> Decimal:
        """Create a new Decimal instance from float f.  Unlike the Decimal.from_float()
        class method, this function observes the context limits.

        """

    def abs(self, x: _Decimal, /) -> Decimal:
        """Return the absolute value of x."""

    def add(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Return the sum of x and y."""

    def canonical(self, x: Decimal, /) -> Decimal:
        """Return a new instance of x."""

    def compare(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Compare x and y numerically."""

    def compare_signal(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Compare x and y numerically.  All NaNs signal."""

    def compare_total(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Compare x and y using their abstract representation."""

    def compare_total_mag(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Compare x and y using their abstract representation, ignoring sign."""

    def copy_abs(self, x: _Decimal, /) -> Decimal:
        """Return a copy of x with the sign set to 0."""

    def copy_decimal(self, x: _Decimal, /) -> Decimal:
        """Return a copy of Decimal x."""

    def copy_negate(self, x: _Decimal, /) -> Decimal:
        """Return a copy of x with the sign inverted."""

    def copy_sign(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Copy the sign from y to x."""

    def divide(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Return x divided by y."""

    def divide_int(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Return x divided by y, truncated to an integer."""

    def divmod(self, x: _Decimal, y: _Decimal, /) -> tuple[Decimal, Decimal]:
        """Return quotient and remainder of the division x / y."""

    def exp(self, x: _Decimal, /) -> Decimal:
        """Return e ** x."""

    def fma(self, x: _Decimal, y: _Decimal, z: _Decimal, /) -> Decimal:
        """Return x multiplied by y, plus z."""

    def is_canonical(self, x: _Decimal, /) -> bool:
        """Return True if x is canonical, False otherwise."""

    def is_finite(self, x: _Decimal, /) -> bool:
        """Return True if x is finite, False otherwise."""

    def is_infinite(self, x: _Decimal, /) -> bool:
        """Return True if x is infinite, False otherwise."""

    def is_nan(self, x: _Decimal, /) -> bool:
        """Return True if x is a qNaN or sNaN, False otherwise."""

    def is_normal(self, x: _Decimal, /) -> bool:
        """Return True if x is a normal number, False otherwise."""

    def is_qnan(self, x: _Decimal, /) -> bool:
        """Return True if x is a quiet NaN, False otherwise."""

    def is_signed(self, x: _Decimal, /) -> bool:
        """Return True if x is negative, False otherwise."""

    def is_snan(self, x: _Decimal, /) -> bool:
        """Return True if x is a signaling NaN, False otherwise."""

    def is_subnormal(self, x: _Decimal, /) -> bool:
        """Return True if x is subnormal, False otherwise."""

    def is_zero(self, x: _Decimal, /) -> bool:
        """Return True if x is a zero, False otherwise."""

    def ln(self, x: _Decimal, /) -> Decimal:
        """Return the natural (base e) logarithm of x."""

    def log10(self, x: _Decimal, /) -> Decimal:
        """Return the base 10 logarithm of x."""

    def logb(self, x: _Decimal, /) -> Decimal:
        """Return the exponent of the magnitude of the operand's MSD."""

    def logical_and(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Digit-wise and of x and y."""

    def logical_invert(self, x: _Decimal, /) -> Decimal:
        """Invert all digits of x."""

    def logical_or(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Digit-wise or of x and y."""

    def logical_xor(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Digit-wise xor of x and y."""

    def max(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Compare the values numerically and return the maximum."""

    def max_mag(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Compare the values numerically with their sign ignored."""

    def min(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Compare the values numerically and return the minimum."""

    def min_mag(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Compare the values numerically with their sign ignored."""

    def minus(self, x: _Decimal, /) -> Decimal:
        """Minus corresponds to the unary prefix minus operator in Python, but applies
        the context to the result.

        """

    def multiply(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Return the product of x and y."""

    def next_minus(self, x: _Decimal, /) -> Decimal:
        """Return the largest representable number smaller than x."""

    def next_plus(self, x: _Decimal, /) -> Decimal:
        """Return the smallest representable number larger than x."""

    def next_toward(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Return the number closest to x, in the direction towards y."""

    def normalize(self, x: _Decimal, /) -> Decimal:
        """Reduce x to its simplest form. Alias for reduce(x)."""

    def number_class(self, x: _Decimal, /) -> str:
        """Return an indication of the class of x."""

    def plus(self, x: _Decimal, /) -> Decimal:
        """Plus corresponds to the unary prefix plus operator in Python, but applies
        the context to the result.

        """

    def power(self, a: _Decimal, b: _Decimal, modulo: _Decimal | None = None) -> Decimal:
        """Compute a**b. If 'a' is negative, then 'b' must be integral. The result
        will be inexact unless 'a' is integral and the result is finite and can
        be expressed exactly in 'precision' digits.  In the Python version the
        result is always correctly rounded, in the C version the result is almost
        always correctly rounded.

        If modulo is given, compute (a**b) % modulo. The following restrictions
        hold:

            * all three arguments must be integral
            * 'b' must be nonnegative
            * at least one of 'a' or 'b' must be nonzero
            * modulo must be nonzero and less than 10**prec in absolute value


        """

    def quantize(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Return a value equal to x (rounded), having the exponent of y."""

    def radix(self) -> Decimal:
        """Return 10."""

    def remainder(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Return the remainder from integer division.  The sign of the result,
        if non-zero, is the same as that of the original dividend.

        """

    def remainder_near(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Return x - y * n, where n is the integer nearest the exact value of x / y
        (if the result is 0 then its sign will be the sign of x).

        """

    def rotate(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Return a copy of x, rotated by y places."""

    def same_quantum(self, x: _Decimal, y: _Decimal, /) -> bool:
        """Return True if the two operands have the same exponent."""

    def scaleb(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Return the first operand after adding the second value to its exp."""

    def shift(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Return a copy of x, shifted by y places."""

    def sqrt(self, x: _Decimal, /) -> Decimal:
        """Square root of a non-negative number to context precision."""

    def subtract(self, x: _Decimal, y: _Decimal, /) -> Decimal:
        """Return the difference between x and y."""

    def to_eng_string(self, x: _Decimal, /) -> str:
        """Convert a number to a string, using engineering notation."""

    def to_sci_string(self, x: _Decimal, /) -> str:
        """Convert a number to a string using scientific notation."""

    def to_integral_exact(self, x: _Decimal, /) -> Decimal:
        """Round to an integer. Signal if the result is rounded or inexact."""

    def to_integral_value(self, x: _Decimal, /) -> Decimal:
        """Round to an integer."""

    def to_integral(self, x: _Decimal, /) -> Decimal:
        """Identical to to_integral_value(x)."""



"""A class supporting chat-style (command/response) protocols.

This class adds support for 'chat' style protocols - where one side
sends a 'command', and the other sends a response (examples would be
the common internet protocols - smtp, nntp, ftp, etc..).

The handle_read() method looks at the input stream for the current
'terminator' (usually '\\r\\n' for single-line responses, '\\r\\n.\\r\\n'
for multi-line output), calling self.found_terminator() on its
receipt.

for example:
Say you build an async nntp client using this class.  At the start
of the connection, you'll have self.terminator set to '\\r\\n', in
order to process the single-line greeting.  Just before issuing a
'LIST' command you'll set it to '\\r\\n.\\r\\n'.  The output of the LIST
command will be accumulated (using your own 'collect_incoming_data'
method) up to the terminator, and then control will be returned to
you - by calling your self.found_terminator() method.
"""

import asyncore
from abc import abstractmethod

class simple_producer:
    def __init__(self, data: bytes, buffer_size: int = 512) -> None: ...
    def more(self) -> bytes: ...

class async_chat(asyncore.dispatcher):
    """This is an abstract class.  You must derive from this class, and add
    the two methods collect_incoming_data() and found_terminator()
    """

    ac_in_buffer_size: int
    ac_out_buffer_size: int
    @abstractmethod
    def collect_incoming_data(self, data: bytes) -> None: ...
    @abstractmethod
    def found_terminator(self) -> None: ...
    def set_terminator(self, term: bytes | int | None) -> None:
        """Set the input delimiter.

        Can be a fixed string of any length, an integer, or None.
        """

    def get_terminator(self) -> bytes | int | None: ...
    def push(self, data: bytes) -> None: ...
    def push_with_producer(self, producer: simple_producer) -> None: ...
    def close_when_done(self) -> None:
        """automatically close this channel once the outgoing queue is empty"""

    def initiate_send(self) -> None: ...
    def discard_buffers(self) -> None: ...



"""This module provides access to mathematical functions for complex
numbers.
"""

from typing import Final, SupportsComplex, SupportsFloat, SupportsIndex
from typing_extensions import TypeAlias

e: Final[float]
pi: Final[float]
inf: Final[float]
infj: Final[complex]
nan: Final[float]
nanj: Final[complex]
tau: Final[float]

_C: TypeAlias = SupportsFloat | SupportsComplex | SupportsIndex | complex

def acos(z: _C, /) -> complex:
    """Return the arc cosine of z."""

def acosh(z: _C, /) -> complex:
    """Return the inverse hyperbolic cosine of z."""

def asin(z: _C, /) -> complex:
    """Return the arc sine of z."""

def asinh(z: _C, /) -> complex:
    """Return the inverse hyperbolic sine of z."""

def atan(z: _C, /) -> complex:
    """Return the arc tangent of z."""

def atanh(z: _C, /) -> complex:
    """Return the inverse hyperbolic tangent of z."""

def cos(z: _C, /) -> complex:
    """Return the cosine of z."""

def cosh(z: _C, /) -> complex:
    """Return the hyperbolic cosine of z."""

def exp(z: _C, /) -> complex:
    """Return the exponential value e**z."""

def isclose(a: _C, b: _C, *, rel_tol: SupportsFloat = 1e-09, abs_tol: SupportsFloat = 0.0) -> bool:
    """Determine whether two complex numbers are close in value.

      rel_tol
        maximum difference for being considered "close", relative to the
        magnitude of the input values
      abs_tol
        maximum difference for being considered "close", regardless of the
        magnitude of the input values

    Return True if a is close in value to b, and False otherwise.

    For the values to be considered close, the difference between them must be
    smaller than at least one of the tolerances.

    -inf, inf and NaN behave similarly to the IEEE 754 Standard. That is, NaN is
    not close to anything, even itself. inf and -inf are only close to themselves.
    """

def isinf(z: _C, /) -> bool:
    """Checks if the real or imaginary part of z is infinite."""

def isnan(z: _C, /) -> bool:
    """Checks if the real or imaginary part of z not a number (NaN)."""

def log(x: _C, base: _C = ..., /) -> complex:
    """log(z[, base]) -> the logarithm of z to the given base.

    If the base is not specified, returns the natural logarithm (base e) of z.
    """

def log10(z: _C, /) -> complex:
    """Return the base-10 logarithm of z."""

def phase(z: _C, /) -> float:
    """Return argument, also known as the phase angle, of a complex."""

def polar(z: _C, /) -> tuple[float, float]:
    """Convert a complex from rectangular coordinates to polar coordinates.

    r is the distance from 0 and phi the phase angle.
    """

def rect(r: float, phi: float, /) -> complex:
    """Convert from polar coordinates to rectangular coordinates."""

def sin(z: _C, /) -> complex:
    """Return the sine of z."""

def sinh(z: _C, /) -> complex:
    """Return the hyperbolic sine of z."""

def sqrt(z: _C, /) -> complex:
    """Return the square root of z."""

def tan(z: _C, /) -> complex:
    """Return the tangent of z."""

def tanh(z: _C, /) -> complex:
    """Return the hyperbolic tangent of z."""

def isfinite(z: _C, /) -> bool:
    """Return True if both the real and imaginary parts of z are finite, else False."""



import sys
from _typeshed import ReadableBuffer
from collections.abc import Mapping, Sequence
from typing import Any, Final, final
from typing_extensions import Self, TypeAlias

_FilterChain: TypeAlias = Sequence[Mapping[str, Any]]

FORMAT_AUTO: Final = 0
FORMAT_XZ: Final = 1
FORMAT_ALONE: Final = 2
FORMAT_RAW: Final = 3
CHECK_NONE: Final = 0
CHECK_CRC32: Final = 1
CHECK_CRC64: Final = 4
CHECK_SHA256: Final = 10
CHECK_ID_MAX: Final = 15
CHECK_UNKNOWN: Final = 16
FILTER_LZMA1: Final[int]  # v big number
FILTER_LZMA2: Final = 33
FILTER_DELTA: Final = 3
FILTER_X86: Final = 4
FILTER_IA64: Final = 6
FILTER_ARM: Final = 7
FILTER_ARMTHUMB: Final = 8
FILTER_SPARC: Final = 9
FILTER_POWERPC: Final = 5
MF_HC3: Final = 3
MF_HC4: Final = 4
MF_BT2: Final = 18
MF_BT3: Final = 19
MF_BT4: Final = 20
MODE_FAST: Final = 1
MODE_NORMAL: Final = 2
PRESET_DEFAULT: Final = 6
PRESET_EXTREME: Final[int]  # v big number

@final
class LZMADecompressor:
    """Create a decompressor object for decompressing data incrementally.

      format
        Specifies the container format of the input stream.  If this is
        FORMAT_AUTO (the default), the decompressor will automatically detect
        whether the input is FORMAT_XZ or FORMAT_ALONE.  Streams created with
        FORMAT_RAW cannot be autodetected.
      memlimit
        Limit the amount of memory used by the decompressor.  This will cause
        decompression to fail if the input cannot be decompressed within the
        given limit.
      filters
        A custom filter chain.  This argument is required for FORMAT_RAW, and
        not accepted with any other format.  When provided, this should be a
        sequence of dicts, each indicating the ID and options for a single
        filter.

    For one-shot decompression, use the decompress() function instead.
    """

    if sys.version_info >= (3, 12):
        def __new__(cls, format: int = 0, memlimit: int | None = None, filters: _FilterChain | None = None) -> Self: ...
    else:
        def __init__(self, format: int = 0, memlimit: int | None = None, filters: _FilterChain | None = None) -> None: ...

    def decompress(self, data: ReadableBuffer, max_length: int = -1) -> bytes:
        """Decompress *data*, returning uncompressed data as bytes.

        If *max_length* is nonnegative, returns at most *max_length* bytes of
        decompressed data. If this limit is reached and further output can be
        produced, *self.needs_input* will be set to ``False``. In this case, the next
        call to *decompress()* may provide *data* as b'' to obtain more of the output.

        If all of the input data was decompressed and returned (either because this
        was less than *max_length* bytes, or because *max_length* was negative),
        *self.needs_input* will be set to True.

        Attempting to decompress data after the end of stream is reached raises an
        EOFError.  Any data found after the end of the stream is ignored and saved in
        the unused_data attribute.
        """

    @property
    def check(self) -> int:
        """ID of the integrity check used by the input stream."""

    @property
    def eof(self) -> bool:
        """True if the end-of-stream marker has been reached."""

    @property
    def unused_data(self) -> bytes:
        """Data found after the end of the compressed stream."""

    @property
    def needs_input(self) -> bool:
        """True if more input is needed before more decompressed data can be produced."""

@final
class LZMACompressor:
    """LZMACompressor(format=FORMAT_XZ, check=-1, preset=None, filters=None)

    Create a compressor object for compressing data incrementally.

    format specifies the container format to use for the output. This can
    be FORMAT_XZ (default), FORMAT_ALONE, or FORMAT_RAW.

    check specifies the integrity check to use. For FORMAT_XZ, the default
    is CHECK_CRC64. FORMAT_ALONE and FORMAT_RAW do not support integrity
    checks; for these formats, check must be omitted, or be CHECK_NONE.

    The settings used by the compressor can be specified either as a
    preset compression level (with the 'preset' argument), or in detail
    as a custom filter chain (with the 'filters' argument). For FORMAT_XZ
    and FORMAT_ALONE, the default is to use the PRESET_DEFAULT preset
    level. For FORMAT_RAW, the caller must always specify a filter chain;
    the raw compressor does not support preset compression levels.

    preset (if provided) should be an integer in the range 0-9, optionally
    OR-ed with the constant PRESET_EXTREME.

    filters (if provided) should be a sequence of dicts. Each dict should
    have an entry for "id" indicating the ID of the filter, plus
    additional entries for options to the filter.

    For one-shot compression, use the compress() function instead.
    """

    if sys.version_info >= (3, 12):
        def __new__(
            cls, format: int = 1, check: int = -1, preset: int | None = None, filters: _FilterChain | None = None
        ) -> Self: ...
    else:
        def __init__(
            self, format: int = 1, check: int = -1, preset: int | None = None, filters: _FilterChain | None = None
        ) -> None: ...

    def compress(self, data: ReadableBuffer, /) -> bytes:
        """Provide data to the compressor object.

        Returns a chunk of compressed data if possible, or b'' otherwise.

        When you have finished providing data to the compressor, call the
        flush() method to finish the compression process.
        """

    def flush(self) -> bytes:
        """Finish the compression process.

        Returns the compressed data left in internal buffers.

        The compressor object may not be used after this method is called.
        """

class LZMAError(Exception):
    """Call to liblzma failed."""

def is_check_supported(check_id: int, /) -> bool:
    """Test whether the given integrity check is supported.

    Always returns True for CHECK_NONE and CHECK_CRC32.
    """



"""faulthandler module."""

import sys
from _typeshed import FileDescriptorLike

def cancel_dump_traceback_later() -> None:
    """Cancel the previous call to dump_traceback_later()."""

def disable() -> None:
    """Disable the fault handler."""

def dump_traceback(file: FileDescriptorLike = ..., all_threads: bool = ...) -> None:
    """Dump the traceback of the current thread, or of all threads if all_threads is True, into file."""

if sys.version_info >= (3, 14):
    def dump_c_stack(file: FileDescriptorLike = ...) -> None:
        """Dump the C stack of the current thread."""

def dump_traceback_later(timeout: float, repeat: bool = ..., file: FileDescriptorLike = ..., exit: bool = ...) -> None:
    """Dump the traceback of all threads in timeout seconds,
    or each timeout seconds if repeat is True. If exit is True, call _exit(1) which is not safe.
    """

if sys.version_info >= (3, 14):
    def enable(file: FileDescriptorLike = ..., all_threads: bool = ..., c_stack: bool = True) -> None:
        """Enable the fault handler."""

else:
    def enable(file: FileDescriptorLike = ..., all_threads: bool = ...) -> None:
        """Enable the fault handler."""

def is_enabled() -> bool:
    """Check if the handler is enabled."""

if sys.platform != "win32":
    def register(signum: int, file: FileDescriptorLike = ..., all_threads: bool = ..., chain: bool = ...) -> None:
        """Register a handler for the signal 'signum': dump the traceback of the current thread, or of all threads if all_threads is True, into file."""

    def unregister(signum: int, /) -> None:
        """Unregister the handler of the signal 'signum' registered by register()."""



"""Python part of the warnings subsystem."""

import re
import sys
from _warnings import warn as warn, warn_explicit as warn_explicit
from collections.abc import Sequence
from types import ModuleType, TracebackType
from typing import Any, Generic, Literal, TextIO, overload
from typing_extensions import LiteralString, TypeAlias, TypeVar

__all__ = [
    "warn",
    "warn_explicit",
    "showwarning",
    "formatwarning",
    "filterwarnings",
    "simplefilter",
    "resetwarnings",
    "catch_warnings",
]

if sys.version_info >= (3, 13):
    __all__ += ["deprecated"]

_T = TypeVar("_T")
_W_co = TypeVar("_W_co", bound=list[WarningMessage] | None, default=list[WarningMessage] | None, covariant=True)

if sys.version_info >= (3, 14):
    _ActionKind: TypeAlias = Literal["default", "error", "ignore", "always", "module", "once"]
else:
    _ActionKind: TypeAlias = Literal["default", "error", "ignore", "always", "all", "module", "once"]
filters: Sequence[tuple[str, re.Pattern[str] | None, type[Warning], re.Pattern[str] | None, int]]  # undocumented, do not mutate

def showwarning(
    message: Warning | str,
    category: type[Warning],
    filename: str,
    lineno: int,
    file: TextIO | None = None,
    line: str | None = None,
) -> None:
    """Hook to write a warning to a file; replace if you like."""

def formatwarning(message: Warning | str, category: type[Warning], filename: str, lineno: int, line: str | None = None) -> str:
    """Function to format a warning the standard way."""

def filterwarnings(
    action: _ActionKind, message: str = "", category: type[Warning] = ..., module: str = "", lineno: int = 0, append: bool = False
) -> None:
    """Insert an entry into the list of warnings filters (at the front).

    'action' -- one of "error", "ignore", "always", "all", "default", "module",
                or "once"
    'message' -- a regex that the warning message must match
    'category' -- a class that the warning must be a subclass of
    'module' -- a regex that the module name must match
    'lineno' -- an integer line number, 0 matches all warnings
    'append' -- if true, append to the list of filters
    """

def simplefilter(action: _ActionKind, category: type[Warning] = ..., lineno: int = 0, append: bool = False) -> None:
    """Insert a simple entry into the list of warnings filters (at the front).

    A simple filter matches all modules and messages.
    'action' -- one of "error", "ignore", "always", "all", "default", "module",
                or "once"
    'category' -- a class that the warning must be a subclass of
    'lineno' -- an integer line number, 0 matches all warnings
    'append' -- if true, append to the list of filters
    """

def resetwarnings() -> None:
    """Clear the list of warning filters, so that no filters are active."""

class _OptionError(Exception):
    """Exception used by option processing helpers."""

class WarningMessage:
    message: Warning | str
    category: type[Warning]
    filename: str
    lineno: int
    file: TextIO | None
    line: str | None
    source: Any | None
    def __init__(
        self,
        message: Warning | str,
        category: type[Warning],
        filename: str,
        lineno: int,
        file: TextIO | None = None,
        line: str | None = None,
        source: Any | None = None,
    ) -> None: ...

class catch_warnings(Generic[_W_co]):
    """A context manager that copies and restores the warnings filter upon
    exiting the context.

    The 'record' argument specifies whether warnings should be captured by a
    custom implementation of warnings.showwarning() and be appended to a list
    returned by the context manager. Otherwise None is returned by the context
    manager. The objects appended to the list are arguments whose attributes
    mirror the arguments to showwarning().

    The 'module' argument is to specify an alternative module to the module
    named 'warnings' and imported under that name. This argument is only useful
    when testing the warnings module itself.

    If the 'action' argument is not None, the remaining arguments are passed
    to warnings.simplefilter() as if it were called immediately on entering the
    context.
    """

    if sys.version_info >= (3, 11):
        @overload
        def __init__(
            self: catch_warnings[None],
            *,
            record: Literal[False] = False,
            module: ModuleType | None = None,
            action: _ActionKind | None = None,
            category: type[Warning] = ...,
            lineno: int = 0,
            append: bool = False,
        ) -> None:
            """Specify whether to record warnings and if an alternative module
            should be used other than sys.modules['warnings'].

            """

        @overload
        def __init__(
            self: catch_warnings[list[WarningMessage]],
            *,
            record: Literal[True],
            module: ModuleType | None = None,
            action: _ActionKind | None = None,
            category: type[Warning] = ...,
            lineno: int = 0,
            append: bool = False,
        ) -> None: ...
        @overload
        def __init__(
            self,
            *,
            record: bool,
            module: ModuleType | None = None,
            action: _ActionKind | None = None,
            category: type[Warning] = ...,
            lineno: int = 0,
            append: bool = False,
        ) -> None: ...
    else:
        @overload
        def __init__(self: catch_warnings[None], *, record: Literal[False] = False, module: ModuleType | None = None) -> None:
            """Specify whether to record warnings and if an alternative module
            should be used other than sys.modules['warnings'].

            For compatibility with Python 3.0, please consider all arguments to be
            keyword-only.

            """

        @overload
        def __init__(
            self: catch_warnings[list[WarningMessage]], *, record: Literal[True], module: ModuleType | None = None
        ) -> None: ...
        @overload
        def __init__(self, *, record: bool, module: ModuleType | None = None) -> None: ...

    def __enter__(self) -> _W_co: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

if sys.version_info >= (3, 13):
    class deprecated:
        """Indicate that a class, function or overload is deprecated.

        When this decorator is applied to an object, the type checker
        will generate a diagnostic on usage of the deprecated object.

        Usage:

            @deprecated("Use B instead")
            class A:
                pass

            @deprecated("Use g instead")
            def f():
                pass

            @overload
            @deprecated("int support is deprecated")
            def g(x: int) -> int: ...
            @overload
            def g(x: str) -> int: ...

        The warning specified by *category* will be emitted at runtime
        on use of deprecated objects. For functions, that happens on calls;
        for classes, on instantiation and on creation of subclasses.
        If the *category* is ``None``, no warning is emitted at runtime.
        The *stacklevel* determines where the
        warning is emitted. If it is ``1`` (the default), the warning
        is emitted at the direct caller of the deprecated object; if it
        is higher, it is emitted further up the stack.
        Static type checker behavior is not affected by the *category*
        and *stacklevel* arguments.

        The deprecation message passed to the decorator is saved in the
        ``__deprecated__`` attribute on the decorated object.
        If applied to an overload, the decorator
        must be after the ``@overload`` decorator for the attribute to
        exist on the overload as returned by ``get_overloads()``.

        See PEP 702 for details.

        """

        message: LiteralString
        category: type[Warning] | None
        stacklevel: int
        def __init__(self, message: LiteralString, /, *, category: type[Warning] | None = ..., stacklevel: int = 1) -> None: ...
        def __call__(self, arg: _T, /) -> _T: ...



def func():
    pass
# fmt: off
x = 1
# fmt: on


# fmt: off
def func():
    pass
# fmt: on
x = 1


# fmt: off
def func():
    pass
# fmt: on
def func():
    pass


# fmt: off
def func():
    pass
# fmt: off
def func():
    pass


# fmt: on
def func():
    pass
# fmt: on
def func():
    pass



"""A POP3 client class.

Based on the J. Myers POP3 draft, Jan. 96
"""

import socket
import ssl
import sys
from builtins import list as _list  # conflicts with a method named "list"
from re import Pattern
from typing import Any, BinaryIO, Final, NoReturn, overload
from typing_extensions import TypeAlias

__all__ = ["POP3", "error_proto", "POP3_SSL"]

_LongResp: TypeAlias = tuple[bytes, list[bytes], int]

class error_proto(Exception): ...

POP3_PORT: Final = 110
POP3_SSL_PORT: Final = 995
CR: Final = b"\r"
LF: Final = b"\n"
CRLF: Final = b"\r\n"
HAVE_SSL: Final[bool]

class POP3:
    """This class supports both the minimal and optional command sets.
    Arguments can be strings or integers (where appropriate)
    (e.g.: retr(1) and retr('1') both work equally well.

    Minimal Command Set:
            USER name               user(name)
            PASS string             pass_(string)
            STAT                    stat()
            LIST [msg]              list(msg = None)
            RETR msg                retr(msg)
            DELE msg                dele(msg)
            NOOP                    noop()
            RSET                    rset()
            QUIT                    quit()

    Optional Commands (some servers support these):
            RPOP name               rpop(name)
            APOP name digest        apop(name, digest)
            TOP msg n               top(msg, n)
            UIDL [msg]              uidl(msg = None)
            CAPA                    capa()
            STLS                    stls()
            UTF8                    utf8()

    Raises one exception: 'error_proto'.

    Instantiate with:
            POP3(hostname, port=110)

    NB:     the POP protocol locks the mailbox from user
            authorization until QUIT, so be sure to get in, suck
            the messages, and quit, each time you access the
            mailbox.

            POP is a line-based protocol, which means large mail
            messages consume lots of python cycles reading them
            line-by-line.

            If it's available on your mail server, use IMAP4
            instead, it doesn't suffer from the two problems
            above.
    """

    encoding: str
    host: str
    port: int
    sock: socket.socket
    file: BinaryIO
    welcome: bytes
    def __init__(self, host: str, port: int = 110, timeout: float = ...) -> None: ...
    def getwelcome(self) -> bytes: ...
    def set_debuglevel(self, level: int) -> None: ...
    def user(self, user: str) -> bytes:
        """Send user name, return response

        (should indicate password required).
        """

    def pass_(self, pswd: str) -> bytes:
        """Send password, return response

        (response includes message count, mailbox size).

        NB: mailbox is locked by server from here to 'quit()'
        """

    def stat(self) -> tuple[int, int]:
        """Get mailbox status.

        Result is tuple of 2 ints (message count, mailbox size)
        """

    def list(self, which: Any | None = None) -> _LongResp:
        """Request listing, return result.

        Result without a message number argument is in form
        ['response', ['mesg_num octets', ...], octets].

        Result when a message number argument is given is a
        single response: the "scan listing" for that message.
        """

    def retr(self, which: Any) -> _LongResp:
        """Retrieve whole message number 'which'.

        Result is in form ['response', ['line', ...], octets].
        """

    def dele(self, which: Any) -> bytes:
        """Delete message number 'which'.

        Result is 'response'.
        """

    def noop(self) -> bytes:
        """Does nothing.

        One supposes the response indicates the server is alive.
        """

    def rset(self) -> bytes:
        """Unmark all messages marked for deletion."""

    def quit(self) -> bytes:
        """Signoff: commit changes on server, unlock mailbox, close connection."""

    def close(self) -> None:
        """Close the connection without assuming anything about it."""

    def rpop(self, user: str) -> bytes:
        """Send RPOP command to access the mailbox with an alternate user."""
    timestamp: Pattern[str]
    def apop(self, user: str, password: str) -> bytes:
        """Authorisation

        - only possible if server has supplied a timestamp in initial greeting.

        Args:
                user     - mailbox user;
                password - mailbox password.

        NB: mailbox is locked by server from here to 'quit()'
        """

    def top(self, which: Any, howmuch: int) -> _LongResp:
        """Retrieve message header of message number 'which'
        and first 'howmuch' lines of message body.

        Result is in form ['response', ['line', ...], octets].
        """

    @overload
    def uidl(self) -> _LongResp:
        """Return message digest (unique id) list.

        If 'which', result contains unique id for that message
        in the form 'response mesgnum uid', otherwise result is
        the list ['response', ['mesgnum uid', ...], octets]
        """

    @overload
    def uidl(self, which: Any) -> bytes: ...
    def utf8(self) -> bytes:
        """Try to enter UTF-8 mode (see RFC 6856). Returns server response."""

    def capa(self) -> dict[str, _list[str]]:
        """Return server capabilities (RFC 2449) as a dictionary
        >>> c=poplib.POP3('localhost')
        >>> c.capa()
        {'IMPLEMENTATION': ['Cyrus', 'POP3', 'server', 'v2.2.12'],
         'TOP': [], 'LOGIN-DELAY': ['0'], 'AUTH-RESP-CODE': [],
         'EXPIRE': ['NEVER'], 'USER': [], 'STLS': [], 'PIPELINING': [],
         'UIDL': [], 'RESP-CODES': []}
        >>>

        Really, according to RFC 2449, the cyrus folks should avoid
        having the implementation split into multiple arguments...
        """

    def stls(self, context: ssl.SSLContext | None = None) -> bytes:
        """Start a TLS session on the active connection as specified in RFC 2595.

        context - a ssl.SSLContext
        """

class POP3_SSL(POP3):
    """POP3 client class over SSL connection

    Instantiate with: POP3_SSL(hostname, port=995, context=None)

           hostname - the hostname of the pop3 over ssl server
           port - port number
           context - a ssl.SSLContext

    See the methods of the parent class POP3 for more documentation.
    """

    if sys.version_info >= (3, 12):
        def __init__(
            self, host: str, port: int = 995, *, timeout: float = ..., context: ssl.SSLContext | None = None
        ) -> None: ...
        def stls(self, context: Any = None) -> NoReturn:
            """The method unconditionally raises an exception since the
            STLS command doesn't make any sense on an already established
            SSL/TLS session.
            """
    else:
        def __init__(
            self,
            host: str,
            port: int = 995,
            keyfile: str | None = None,
            certfile: str | None = None,
            timeout: float = ...,
            context: ssl.SSLContext | None = None,
        ) -> None: ...
        # "context" is actually the last argument,
        # but that breaks LSP and it doesn't really matter because all the arguments are ignored
        def stls(self, context: Any = None, keyfile: Any = None, certfile: Any = None) -> NoReturn:
            """The method unconditionally raises an exception since the
            STLS command doesn't make any sense on an already established
            SSL/TLS session.
            """



# Based on Python-3.4.3/Lib/test/test_scope.py

def top_method(self):

    def outer():
        class Test:
            def actual_global(self):
                return str("global")
            def str(self):
                return str(self)



import sys
from collections.abc import Iterable
from enum import Enum
from typing import (
    Any,
    ClassVar,
    LiteralString,
    Protocol,
    _SpecialForm,
)

from typing_extensions import Self  # noqa: UP035

# Special operations
def static_assert(condition: object, msg: LiteralString | None = None) -> None: ...

# Types
Unknown = object()
AlwaysTruthy = object()
AlwaysFalsy = object()

# Special forms
Not: _SpecialForm
Intersection: _SpecialForm
TypeOf: _SpecialForm
CallableTypeOf: _SpecialForm
# Top[T] evaluates to the top materialization of T, a type that is a supertype
# of every materialization of T.
Top: _SpecialForm
# Bottom[T] evaluates to the bottom materialization of T, a type that is a subtype
# of every materialization of T.
Bottom: _SpecialForm

# ty treats annotations of `float` to mean `float | int`, and annotations of `complex`
# to mean `complex | float | int`. This is to support a typing-system special case [1].
# We therefore provide `JustFloat` and `JustComplex` to represent the "bare" `float` and
# `complex` types, respectively.
#
# [1]: https://typing.readthedocs.io/en/latest/spec/special-types.html#special-cases-for-float-and-complex
type JustFloat = TypeOf[1.0]
type JustComplex = TypeOf[1.0j]

# Constraints
class ConstraintSet:
    def __bool__(self) -> bool: ...
    def __and__(self, other: ConstraintSet) -> ConstraintSet: ...
    def __or__(self, other: ConstraintSet) -> ConstraintSet: ...
    def __invert__(self) -> ConstraintSet: ...

def range_constraint(
    lower_bound: Any, typevar: Any, upper_bound: Any
) -> ConstraintSet: ...
def negated_range_constraint(
    lower_bound: Any, typevar: Any, upper_bound: Any
) -> ConstraintSet: ...

# Predicates on types
#
# Ideally, these would be annotated using `TypeForm`, but that has not been
# standardized yet (https://peps.python.org/pep-0747).
def is_equivalent_to(type_a: Any, type_b: Any) -> ConstraintSet: ...
def is_subtype_of(type_a: Any, type_b: Any) -> ConstraintSet: ...
def is_assignable_to(type_a: Any, type_b: Any) -> ConstraintSet: ...
def is_disjoint_from(type_a: Any, type_b: Any) -> ConstraintSet: ...
def is_singleton(ty: Any) -> bool: ...
def is_single_valued(ty: Any) -> bool: ...

# Returns the generic context of a type as a tuple of typevars, or `None` if the
# type is not generic.
def generic_context(ty: Any) -> Any: ...

# Returns the `__all__` names of a module as a tuple of sorted strings, or `None` if
# either the module does not have `__all__` or it has invalid elements.
def dunder_all_names(module: Any) -> Any: ...

# List all members of an enum.
def enum_members[E: type[Enum]](enum: E) -> tuple[str, ...]: ...

# Returns a tuple of all members of the given object, similar to `dir(obj)` and
# `inspect.getmembers(obj)`, with at least the following differences:
#
# * `dir` and `inspect.getmembers` may use runtime mutable state to construct
# the list of attributes returned. In contrast, this routine is limited to
# static information only.
# * `dir` will respect an object's `__dir__` implementation, if present, but
# this method (currently) does not.
def all_members(obj: Any) -> tuple[str, ...]: ...

# Returns `True` if the given object has a member with the given name.
def has_member(obj: Any, name: str) -> bool: ...

# Passing a protocol type to this function will cause ty to emit an info-level
# diagnostic describing the protocol's interface. Passing a non-protocol type
# will cause ty to emit an error diagnostic.
def reveal_protocol_interface(protocol: type) -> None: ...

# A protocol describing an interface that should be satisfied by all named tuples
# created using `typing.NamedTuple` or `collections.namedtuple`.
class NamedTupleLike(Protocol):
    # from typing.NamedTuple stub
    _field_defaults: ClassVar[dict[str, Any]]
    _fields: ClassVar[tuple[str, ...]]
    @classmethod
    def _make(cls: type[Self], iterable: Iterable[Any]) -> Self: ...
    def _asdict(self, /) -> dict[str, Any]: ...
    def _replace(self, /, **kwargs) -> Self: ...
    if sys.version_info >= (3, 13):
        def __replace__(self, **kwargs) -> Self: ...



"""Append module search paths for third-party packages to sys.path.

****************************************************************
* This module is automatically imported during initialization. *
****************************************************************

This will append site-specific paths to the module search path.  On
Unix (including Mac OSX), it starts with sys.prefix and
sys.exec_prefix (if different) and appends
lib/python<version>/site-packages.
On other platforms (such as Windows), it tries each of the
prefixes directly, as well as with lib/site-packages appended.  The
resulting directories, if they exist, are appended to sys.path, and
also inspected for path configuration files.

If a file named "pyvenv.cfg" exists one directory above sys.executable,
sys.prefix and sys.exec_prefix are set to that directory and
it is also checked for site-packages (sys.base_prefix and
sys.base_exec_prefix will always be the "real" prefixes of the Python
installation). If "pyvenv.cfg" (a bootstrap configuration file) contains
the key "include-system-site-packages" set to anything other than "false"
(case-insensitive), the system-level prefixes will still also be
searched for site-packages; otherwise they won't.

All of the resulting site-specific directories, if they exist, are
appended to sys.path, and also inspected for path configuration
files.

A path configuration file is a file whose name has the form
<package>.pth; its contents are additional directories (one per line)
to be added to sys.path.  Non-existing directories (or
non-directories) are never added to sys.path; no directory is added to
sys.path more than once.  Blank lines and lines beginning with
'#' are skipped. Lines starting with 'import' are executed.

For example, suppose sys.prefix and sys.exec_prefix are set to
/usr/local and there is a directory /usr/local/lib/python2.5/site-packages
with three subdirectories, foo, bar and spam, and two path
configuration files, foo.pth and bar.pth.  Assume foo.pth contains the
following:

  # foo package configuration
  foo
  bar
  bletch

and bar.pth contains:

  # bar package configuration
  bar

Then the following directories are added to sys.path, in this order:

  /usr/local/lib/python2.5/site-packages/bar
  /usr/local/lib/python2.5/site-packages/foo

Note that bletch is omitted because it doesn't exist; bar precedes foo
because bar.pth comes alphabetically before foo.pth; and spam is
omitted because it is not mentioned in either path configuration file.

The readline module is also automatically configured to enable
completion for systems that support it.  This can be overridden in
sitecustomize, usercustomize or PYTHONSTARTUP.  Starting Python in
isolated mode (-I) disables automatic readline configuration.

After these operations, an attempt is made to import a module
named sitecustomize, which can perform arbitrary additional
site-specific customizations.  If this import fails with an
ImportError exception, it is silently ignored.
"""

import sys
from _typeshed import StrPath
from collections.abc import Iterable

PREFIXES: list[str]
ENABLE_USER_SITE: bool | None
USER_SITE: str | None
USER_BASE: str | None

def main() -> None:
    """Add standard site-specific directories to the module search path.

    This function is called automatically when this module is imported,
    unless the python interpreter was started with the -S flag.
    """

def abs_paths() -> None:  # undocumented
    """Set all module __file__ and __cached__ attributes to an absolute path"""

def addpackage(sitedir: StrPath, name: StrPath, known_paths: set[str] | None) -> set[str] | None:  # undocumented
    """Process a .pth file within the site-packages directory:
    For each line in the file, either combine it with sitedir to a path
    and add that to known_paths, or execute it if it starts with 'import '.
    """

def addsitedir(sitedir: str, known_paths: set[str] | None = None) -> None:
    """Add 'sitedir' argument to sys.path if missing and handle .pth files in
    'sitedir'
    """

def addsitepackages(known_paths: set[str] | None, prefixes: Iterable[str] | None = None) -> set[str] | None:  # undocumented
    """Add site-packages to sys.path"""

def addusersitepackages(known_paths: set[str] | None) -> set[str] | None:  # undocumented
    """Add a per user site-package to sys.path

    Each user has its own python directory with site-packages in the
    home directory.
    """

def check_enableusersite() -> bool | None:  # undocumented
    """Check if user site directory is safe for inclusion

    The function tests for the command line flag (including environment var),
    process uid/gid equal to effective uid/gid.

    None: Disabled for security reasons
    False: Disabled by user (command line option)
    True: Safe and enabled
    """

if sys.version_info >= (3, 13):
    def gethistoryfile() -> str:  # undocumented
        """Check if the PYTHON_HISTORY environment variable is set and define
        it as the .python_history file.  If PYTHON_HISTORY is not set, use the
        default .python_history file.
        """

def enablerlcompleter() -> None:  # undocumented
    """Enable default readline configuration on interactive prompts, by
    registering a sys.__interactivehook__.
    """

if sys.version_info >= (3, 13):
    def register_readline() -> None:  # undocumented
        """Configure readline completion on interactive prompts.

        If the readline module can be imported, the hook will set the Tab key
        as completion key and register ~/.python_history as history file.
        This can be overridden in the sitecustomize or usercustomize module,
        or in a PYTHONSTARTUP file.
        """

def execsitecustomize() -> None:  # undocumented
    """Run custom site specific code, if available."""

def execusercustomize() -> None:  # undocumented
    """Run custom user specific code, if available."""

def getsitepackages(prefixes: Iterable[str] | None = None) -> list[str]:
    """Returns a list containing all global site-packages directories.

    For each directory present in ``prefixes`` (or the global ``PREFIXES``),
    this function will find its `site-packages` subdirectory depending on the
    system environment, and will return a list of full paths.
    """

def getuserbase() -> str:
    """Returns the `user base` directory path.

    The `user base` directory can be used to store data. If the global
    variable ``USER_BASE`` is not initialized yet, this function will also set
    it.
    """

def getusersitepackages() -> str:
    """Returns the user-specific site-packages directory path.

    If the global variable ``USER_SITE`` is not initialized yet, this
    function will also set it.
    """

def makepath(*paths: StrPath) -> tuple[str, str]: ...  # undocumented
def removeduppaths() -> set[str]:  # undocumented
    """Remove duplicate entries from sys.path along with making them
    absolute
    """

def setcopyright() -> None:  # undocumented
    """Set 'copyright' and 'credits' in builtins"""

def sethelper() -> None: ...  # undocumented
def setquit() -> None:  # undocumented
    """Define new builtins 'quit' and 'exit'.

    These are objects which make the interpreter exit when called.
    The repr of each object contains a hint at how it works.

    """

def venv(known_paths: set[str] | None) -> set[str] | None: ...  # undocumented



"""(Extremely) low-level import machinery bits as used by importlib."""

import sys
import types
from _typeshed import ReadableBuffer
from importlib.machinery import ModuleSpec
from typing import Any

check_hash_based_pycs: str
if sys.version_info >= (3, 14):
    pyc_magic_number_token: int

def source_hash(key: int, source: ReadableBuffer) -> bytes: ...
def create_builtin(spec: ModuleSpec, /) -> types.ModuleType:
    """Create an extension module."""

def create_dynamic(spec: ModuleSpec, file: Any = None, /) -> types.ModuleType:
    """Create an extension module."""

def acquire_lock() -> None:
    """Acquires the interpreter's import lock for the current thread.

    This lock should be used by import hooks to ensure thread-safety when importing
    modules. On platforms without threads, this function does nothing.
    """

def exec_builtin(mod: types.ModuleType, /) -> int:
    """Initialize a built-in module."""

def exec_dynamic(mod: types.ModuleType, /) -> int:
    """Initialize an extension module."""

def extension_suffixes() -> list[str]:
    """Returns the list of file suffixes used to identify extension modules."""

def init_frozen(name: str, /) -> types.ModuleType:
    """Initializes a frozen module."""

def is_builtin(name: str, /) -> int:
    """Returns True if the module name corresponds to a built-in module."""

def is_frozen(name: str, /) -> bool:
    """Returns True if the module name corresponds to a frozen module."""

def is_frozen_package(name: str, /) -> bool:
    """Returns True if the module name is of a frozen package."""

def lock_held() -> bool:
    """Return True if the import lock is currently held, else False.

    On platforms without threads, return False.
    """

def release_lock() -> None:
    """Release the interpreter's import lock.

    On platforms without threads, this function does nothing.
    """

if sys.version_info >= (3, 11):
    def find_frozen(name: str, /, *, withdata: bool = False) -> tuple[memoryview | None, bool, str | None] | None:
        """Return info about the corresponding frozen module (if there is one) or None.

        The returned info (a 2-tuple):

         * data         the raw marshalled bytes
         * is_package   whether or not it is a package
         * origname     the originally frozen module's name, or None if not
                        a stdlib module (this will usually be the same as
                        the module's current name)
        """

    def get_frozen_object(name: str, data: ReadableBuffer | None = None, /) -> types.CodeType:
        """Create a code object for a frozen module."""

else:
    def get_frozen_object(name: str, /) -> types.CodeType:
        """Create a code object for a frozen module."""



"""
Black's `Preview.module_docstring_newlines`
"""
first_stmt_after_module_level_docstring = 1


class CachedRepository:
    # Black's `Preview.dummy_implementations`
    def get_release_info(self): ...


def raw_docstring():

    r"""Black's `Preview.accept_raw_docstrings`
        a
            b
    """
    pass


def reference_docstring_newlines():

    """A regular docstring for comparison
        a
            b
    """
    pass


class RemoveNewlineBeforeClassDocstring:

    """Black's `Preview.no_blank_line_before_class_docstring`"""


def f():
    """Black's `Preview.prefer_splitting_right_hand_side_of_assignments`"""
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa[
        bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
    ] = cccccccc.ccccccccccccc.cccccccc

    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa[
        bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
    ] = cccccccc.ccccccccccccc().cccccccc

    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa[
        bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
    ] = cccccccc.ccccccccccccc(d).cccccccc

    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa[bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb] = (
        cccccccc.ccccccccccccc(d).cccccccc + e
    )

    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa[bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb] = (
        cccccccc.ccccccccccccc.cccccccc + e
    )
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa[bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb] = (
        cccccccc.ccccccccccccc.cccccccc
        + eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee
    )

    self._cache: dict[
        DependencyCacheKey, list[list[DependencyPackage]]
    ] = collections.defaultdict(list)
    self._cached_dependencies_by_level: dict[
        int, list[DependencyCacheKey]
    ] = collections.defaultdict(list)



import sys

def find_library(name: str) -> str | None: ...

if sys.platform == "win32":
    def find_msvcrt() -> str | None:
        """Return the name of the VC runtime dll"""

if sys.version_info >= (3, 14):
    def dllist() -> list[str]:
        """Return a list of loaded shared libraries in the current process."""

def test() -> None: ...




raise ImproperlyConfigured(
    "The app module %r has multiple filesystem locations (%r); "
    "you must configure this app with an AppConfig subclass "
    "with a 'path' class attr                               ibute." % (module, paths)
)

raise ImproperlyConfigured(
    "The app module %r has multiple filesystem locations (%r); "
    "you must configure this app with an AppConfig subclass "
    "with a 'path' class attr                               ibute."
    %
    # comment
    (module, paths)
)

# Only important in parenthesized context because implicit string continuation otherwise doesn't expand
"The app module %r has multiple filesystem locations (%r); " "you must configure this app with an AppConfig subclass " "with a 'path' class attribute." % (
    module,
    paths,
)

("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa" "bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb" "cccccccccccccccccccccccccccccccccccccccccccc" % (aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa, b, c, d))
("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa" "bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb" "cccccccccccccccccccccccccccccccccccccccccccc" % aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa)

def test():
    return (
        "\n%(modified_count)s %(identifier)s %(action)s"
        "%(destination)s%(unmodified)s%(post_processed)s."
    ) % {
        "modified_count": modified_count,
        "identifier": "static file" + ("" if modified_count == 1 else "s"),
        "action": "symlinked" if self.symlink else "copied",
        "destination": (" to '%s'" % destination_path if destination_path else ""),
        "unmodified": (
            ", %s unmodified" % unmodified_count if collected["unmodified"] else ""
        ),
        "post_processed": (
            collected["post_processed"]
            and ", %s post-processed" % post_processed_count
            or ""
        ),
    }

# trailing expression comment
self._assert_skipping(
    SkipTestCase("test_foo").test_foo,
    ValueError,
    "skipUnlessDBFeature cannot be used on test_foo (test_utils.tests."
    "SkippingTestCase.test_skip_unless_db_feature.<locals>.SkipTestCase%s) "
    "as SkippingTestCase.test_skip_unless_db_feature.<locals>.SkipTestCase "
    "doesn't allow queries against the 'default' database."
    # Python 3.11 uses fully qualified test name in the output.
    % (".test_foo" if PY311 else ""),
    )

# dangling operator comment
self._assert_skipping(
    SkipTestCase("test_foo").test_foo,
    ValueError,
    "skipUnlessDBFeature cannot be used on test_foo (test_utils.tests."
    "SkippingTestCase.test_skip_unless_db_feature.<locals>.SkipTestCase%s) "
    "as SkippingTestCase.test_skip_unless_db_feature.<locals>.SkipTestCase "
    "doesn't allow queries against the 'default' database."
    % # Python 3.11 uses fully qualified test name in the output.
    (".test_foo" if PY311 else ""),
    )

# Black keeps as many operands as fit on the same line as the `%`. Ruff does not. This is intentional as these are rare and complicated things significantly
(
    "aaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
    "bbbbbbbbbbbbbbbbbbbbbbbbbbbbb"
    "cccccccccccccccccccccccccc"
    % aaaaaaaaaaaa
    + x
)

(
    b + c + d +
    "aaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
    "bbbbbbbbbbbbbbbbbbbbbbbbbbbbb"
    "cccccccccccccccccccccccccc"
    % aaaaaaaaaaaa
    + x
)

(
    b + c + d +
    "aaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
    f"bbbbbb{z}bbbbbbbbbbbbbbbbbbbbbbb"
    "cccccccccccccccccccccccccc"
    % aaaaaaaaaaaa
    + x
)

(
    b < c > d <
    "aaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
    "bbbbbbbbbbbbbbbbbbbbbbbbbbbbb"
    "cccccccccccccccccccccccccc"
    % aaaaaaaaaaaa
    > x
)


self.assertEqual(
    response.status_code,
    status_code,
    msg_prefix + "Couldn't retrieve content: Response code was %d"
                 " (expected %d)" % (response.status_code, status_code),
)

def test():
    return (
       "((TIME_TO_SEC(%(lhs)s) * 1000000 + MICROSECOND(%(lhs)s)) -"
       " (TIME_TO_SEC(%(rhs)s) * 1000000 + MICROSECOND(%(rhs)s)))"
   ) % {"lhs": lhs_sql, "rhs": rhs_sql}, tuple(lhs_params) * 2 + tuple(rhs_params) * 2

def test2():
    return "RETURNING %s INTO %s" % (
        ", ".join(field_names),
        ", ".join(["%s"] * len(params)),
    ), tuple(params)

def test3():
    return (
               "(CASE WHEN JSON_TYPE(%s, %%s) IN (%s) "
               "THEN JSON_TYPE(%s, %%s) ELSE JSON_EXTRACT(%s, %%s) END)"
           ) % (lhs, datatype_values, lhs, lhs), (tuple(params) + (json_path,)) * 3

c = (a +
    # test leading binary comment
    "a" "b" * b
)

c = (a *
     # test leading comment
     "a" "b" + b
     )

c = (a
     + # test trailing comment
     "a" "b" * b
     )

c = (a
     +
     "a" "b" # test trailing comment
     * b
     )

c = (a
     *
     "a" "b" # test trailing binary comment
     + b
     )

c = (a
     *
     "a" "b"
     + # test trailing operator comment
     b
     )

c = (a
     *
     "a" "b"
     +
     # test trailing operator comment
     b
     )

c = ("a" "b" +
     # test leading binary comment
     "a" "b"
 )

(
    b + c + d +
    "aaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
    "bbbbbbbbbbbbbbbbbbbbbbbbbbbbb" +
    "cccccccccccccccccccccccccc"
    "dddddddddddddddddddddddddd"
    % aaaaaaaaaaaa
    + x
)

"a" "b" "c" + "d" "e" + "f" "g" + "h" "i" "j"
class EC2REPATH:
    f.write ("Pathway name" + "\t" "Database Identifier" + "\t" "Source database" + "\n")




"""Calendar printing functions

Note when comparing these calendars to the ones printed by cal(1): By
default, these calendars have Monday as the first day of the week, and
Sunday as the last (the European convention). Use setfirstweekday() to
set the first day of the week (0=Monday, 6=Sunday).
"""

import datetime
import enum
import sys
from _typeshed import Unused
from collections.abc import Iterable, Sequence
from time import struct_time
from typing import ClassVar, Final
from typing_extensions import TypeAlias

__all__ = [
    "IllegalMonthError",
    "IllegalWeekdayError",
    "setfirstweekday",
    "firstweekday",
    "isleap",
    "leapdays",
    "weekday",
    "monthrange",
    "monthcalendar",
    "prmonth",
    "month",
    "prcal",
    "calendar",
    "timegm",
    "month_name",
    "month_abbr",
    "day_name",
    "day_abbr",
    "Calendar",
    "TextCalendar",
    "HTMLCalendar",
    "LocaleTextCalendar",
    "LocaleHTMLCalendar",
    "weekheader",
]

if sys.version_info >= (3, 10):
    __all__ += ["FRIDAY", "MONDAY", "SATURDAY", "SUNDAY", "THURSDAY", "TUESDAY", "WEDNESDAY"]
if sys.version_info >= (3, 12):
    __all__ += [
        "Day",
        "Month",
        "JANUARY",
        "FEBRUARY",
        "MARCH",
        "APRIL",
        "MAY",
        "JUNE",
        "JULY",
        "AUGUST",
        "SEPTEMBER",
        "OCTOBER",
        "NOVEMBER",
        "DECEMBER",
    ]

_LocaleType: TypeAlias = tuple[str | None, str | None]

class IllegalMonthError(ValueError):
    def __init__(self, month: int) -> None: ...

class IllegalWeekdayError(ValueError):
    def __init__(self, weekday: int) -> None: ...

def isleap(year: int) -> bool:
    """Return True for leap years, False for non-leap years."""

def leapdays(y1: int, y2: int) -> int:
    """Return number of leap years in range [y1, y2).
    Assume y1 <= y2.
    """

def weekday(year: int, month: int, day: int) -> int:
    """Return weekday (0-6 ~ Mon-Sun) for year, month (1-12), day (1-31)."""

def monthrange(year: int, month: int) -> tuple[int, int]:
    """Return weekday of first day of month (0-6 ~ Mon-Sun)
    and number of days (28-31) for year, month.
    """

class Calendar:
    """
    Base calendar class. This class doesn't do any formatting. It simply
    provides data to subclasses.
    """

    firstweekday: int
    def __init__(self, firstweekday: int = 0) -> None: ...
    def getfirstweekday(self) -> int: ...
    def setfirstweekday(self, firstweekday: int) -> None: ...
    def iterweekdays(self) -> Iterable[int]:
        """
        Return an iterator for one week of weekday numbers starting with the
        configured first one.
        """

    def itermonthdates(self, year: int, month: int) -> Iterable[datetime.date]:
        """
        Return an iterator for one month. The iterator will yield datetime.date
        values and will always iterate through complete weeks, so it will yield
        dates outside the specified month.
        """

    def itermonthdays2(self, year: int, month: int) -> Iterable[tuple[int, int]]:
        """
        Like itermonthdates(), but will yield (day number, weekday number)
        tuples. For days outside the specified month the day number is 0.
        """

    def itermonthdays(self, year: int, month: int) -> Iterable[int]:
        """
        Like itermonthdates(), but will yield day numbers. For days outside
        the specified month the day number is 0.
        """

    def monthdatescalendar(self, year: int, month: int) -> list[list[datetime.date]]:
        """
        Return a matrix (list of lists) representing a month's calendar.
        Each row represents a week; week entries are datetime.date values.
        """

    def monthdays2calendar(self, year: int, month: int) -> list[list[tuple[int, int]]]:
        """
        Return a matrix representing a month's calendar.
        Each row represents a week; week entries are
        (day number, weekday number) tuples. Day numbers outside this month
        are zero.
        """

    def monthdayscalendar(self, year: int, month: int) -> list[list[int]]:
        """
        Return a matrix representing a month's calendar.
        Each row represents a week; days outside this month are zero.
        """

    def yeardatescalendar(self, year: int, width: int = 3) -> list[list[list[list[datetime.date]]]]:
        """
        Return the data for the specified year ready for formatting. The return
        value is a list of month rows. Each month row contains up to width months.
        Each month contains between 4 and 6 weeks and each week contains 1-7
        days. Days are datetime.date objects.
        """

    def yeardays2calendar(self, year: int, width: int = 3) -> list[list[list[list[tuple[int, int]]]]]:
        """
        Return the data for the specified year ready for formatting (similar to
        yeardatescalendar()). Entries in the week lists are
        (day number, weekday number) tuples. Day numbers outside this month are
        zero.
        """

    def yeardayscalendar(self, year: int, width: int = 3) -> list[list[list[list[int]]]]:
        """
        Return the data for the specified year ready for formatting (similar to
        yeardatescalendar()). Entries in the week lists are day numbers.
        Day numbers outside this month are zero.
        """

    def itermonthdays3(self, year: int, month: int) -> Iterable[tuple[int, int, int]]:
        """
        Like itermonthdates(), but will yield (year, month, day) tuples.  Can be
        used for dates outside of datetime.date range.
        """

    def itermonthdays4(self, year: int, month: int) -> Iterable[tuple[int, int, int, int]]:
        """
        Like itermonthdates(), but will yield (year, month, day, day_of_week) tuples.
        Can be used for dates outside of datetime.date range.
        """

class TextCalendar(Calendar):
    """
    Subclass of Calendar that outputs a calendar as a simple plain text
    similar to the UNIX program cal.
    """

    def prweek(self, theweek: int, width: int) -> None:
        """
        Print a single week (no newline).
        """

    def formatday(self, day: int, weekday: int, width: int) -> str:
        """
        Returns a formatted day.
        """

    def formatweek(self, theweek: int, width: int) -> str:
        """
        Returns a single week in a string (no newline).
        """

    def formatweekday(self, day: int, width: int) -> str:
        """
        Returns a formatted week day name.
        """

    def formatweekheader(self, width: int) -> str:
        """
        Return a header for a week.
        """

    def formatmonthname(self, theyear: int, themonth: int, width: int, withyear: bool = True) -> str:
        """
        Return a formatted month name.
        """

    def prmonth(self, theyear: int, themonth: int, w: int = 0, l: int = 0) -> None:
        """
        Print a month's calendar.
        """

    def formatmonth(self, theyear: int, themonth: int, w: int = 0, l: int = 0) -> str:
        """
        Return a month's calendar string (multi-line).
        """

    def formatyear(self, theyear: int, w: int = 2, l: int = 1, c: int = 6, m: int = 3) -> str:
        """
        Returns a year's calendar as a multi-line string.
        """

    def pryear(self, theyear: int, w: int = 0, l: int = 0, c: int = 6, m: int = 3) -> None:
        """Print a year's calendar."""

def firstweekday() -> int: ...
def monthcalendar(year: int, month: int) -> list[list[int]]:
    """
    Return a matrix representing a month's calendar.
    Each row represents a week; days outside this month are zero.
    """

def prweek(theweek: int, width: int) -> None:
    """
    Print a single week (no newline).
    """

def week(theweek: int, width: int) -> str:
    """
    Returns a single week in a string (no newline).
    """

def weekheader(width: int) -> str:
    """
    Return a header for a week.
    """

def prmonth(theyear: int, themonth: int, w: int = 0, l: int = 0) -> None:
    """
    Print a month's calendar.
    """

def month(theyear: int, themonth: int, w: int = 0, l: int = 0) -> str:
    """
    Return a month's calendar string (multi-line).
    """

def calendar(theyear: int, w: int = 2, l: int = 1, c: int = 6, m: int = 3) -> str:
    """
    Returns a year's calendar as a multi-line string.
    """

def prcal(theyear: int, w: int = 0, l: int = 0, c: int = 6, m: int = 3) -> None:
    """Print a year's calendar."""

class HTMLCalendar(Calendar):
    """
    This calendar returns complete HTML pages.
    """

    cssclasses: ClassVar[list[str]]
    cssclass_noday: ClassVar[str]
    cssclasses_weekday_head: ClassVar[list[str]]
    cssclass_month_head: ClassVar[str]
    cssclass_month: ClassVar[str]
    cssclass_year: ClassVar[str]
    cssclass_year_head: ClassVar[str]
    def formatday(self, day: int, weekday: int) -> str:
        """
        Return a day as a table cell.
        """

    def formatweek(self, theweek: int) -> str:
        """
        Return a complete week as a table row.
        """

    def formatweekday(self, day: int) -> str:
        """
        Return a weekday name as a table header.
        """

    def formatweekheader(self) -> str:
        """
        Return a header for a week as a table row.
        """

    def formatmonthname(self, theyear: int, themonth: int, withyear: bool = True) -> str:
        """
        Return a month name as a table row.
        """

    def formatmonth(self, theyear: int, themonth: int, withyear: bool = True) -> str:
        """
        Return a formatted month as a table.
        """

    def formatyear(self, theyear: int, width: int = 3) -> str:
        """
        Return a formatted year as a table of tables.
        """

    def formatyearpage(
        self, theyear: int, width: int = 3, css: str | None = "calendar.css", encoding: str | None = None
    ) -> bytes:
        """
        Return a formatted year as a complete HTML page.
        """

class different_locale:
    def __init__(self, locale: _LocaleType) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, *args: Unused) -> None: ...

class LocaleTextCalendar(TextCalendar):
    """
    This class can be passed a locale name in the constructor and will return
    month and weekday names in the specified locale.
    """

    def __init__(self, firstweekday: int = 0, locale: _LocaleType | None = None) -> None: ...

class LocaleHTMLCalendar(HTMLCalendar):
    """
    This class can be passed a locale name in the constructor and will return
    month and weekday names in the specified locale.
    """

    def __init__(self, firstweekday: int = 0, locale: _LocaleType | None = None) -> None: ...
    def formatweekday(self, day: int) -> str: ...
    def formatmonthname(self, theyear: int, themonth: int, withyear: bool = True) -> str: ...

c: TextCalendar

def setfirstweekday(firstweekday: int) -> None: ...
def format(cols: int, colwidth: int = 20, spacing: int = 6) -> str:
    """Prints multi-column formatting for year calendars"""

def formatstring(cols: int, colwidth: int = 20, spacing: int = 6) -> str:
    """Returns a string formatted from n strings, centered within n columns."""

def timegm(tuple: tuple[int, ...] | struct_time) -> int:
    """Unrelated but handy function to calculate Unix timestamp from GMT."""

# Data attributes
day_name: Sequence[str]
day_abbr: Sequence[str]
month_name: Sequence[str]
month_abbr: Sequence[str]

if sys.version_info >= (3, 12):
    class Month(enum.IntEnum):
        JANUARY = 1
        FEBRUARY = 2
        MARCH = 3
        APRIL = 4
        MAY = 5
        JUNE = 6
        JULY = 7
        AUGUST = 8
        SEPTEMBER = 9
        OCTOBER = 10
        NOVEMBER = 11
        DECEMBER = 12

    JANUARY: Final = Month.JANUARY
    FEBRUARY: Final = Month.FEBRUARY
    MARCH: Final = Month.MARCH
    APRIL: Final = Month.APRIL
    MAY: Final = Month.MAY
    JUNE: Final = Month.JUNE
    JULY: Final = Month.JULY
    AUGUST: Final = Month.AUGUST
    SEPTEMBER: Final = Month.SEPTEMBER
    OCTOBER: Final = Month.OCTOBER
    NOVEMBER: Final = Month.NOVEMBER
    DECEMBER: Final = Month.DECEMBER

    class Day(enum.IntEnum):
        MONDAY = 0
        TUESDAY = 1
        WEDNESDAY = 2
        THURSDAY = 3
        FRIDAY = 4
        SATURDAY = 5
        SUNDAY = 6

    MONDAY: Final = Day.MONDAY
    TUESDAY: Final = Day.TUESDAY
    WEDNESDAY: Final = Day.WEDNESDAY
    THURSDAY: Final = Day.THURSDAY
    FRIDAY: Final = Day.FRIDAY
    SATURDAY: Final = Day.SATURDAY
    SUNDAY: Final = Day.SUNDAY
else:
    MONDAY: Final = 0
    TUESDAY: Final = 1
    WEDNESDAY: Final = 2
    THURSDAY: Final = 3
    FRIDAY: Final = 4
    SATURDAY: Final = 5
    SUNDAY: Final = 6

EPOCH: Final = 1970



"""Classes to generate plain text from a message object tree."""

from _typeshed import SupportsWrite
from email.message import Message
from email.policy import Policy
from typing import Any, Generic, TypeVar, overload
from typing_extensions import Self

__all__ = ["Generator", "DecodedGenerator", "BytesGenerator"]

# By default, generators do not have a message policy.
_MessageT = TypeVar("_MessageT", bound=Message[Any, Any], default=Any)

class Generator(Generic[_MessageT]):
    """Generates output from a Message object tree.

    This basic generator writes the message to the given file object as plain
    text.
    """

    maxheaderlen: int | None
    policy: Policy[_MessageT] | None
    @overload
    def __init__(
        self: Generator[Any],  # The Policy of the message is used.
        outfp: SupportsWrite[str],
        mangle_from_: bool | None = None,
        maxheaderlen: int | None = None,
        *,
        policy: None = None,
    ) -> None:
        """Create the generator for message flattening.

        outfp is the output file-like object for writing the message to.  It
        must have a write() method.

        Optional mangle_from_ is a flag that, when True (the default if policy
        is not set), escapes From_ lines in the body of the message by putting
        a '>' in front of them.

        Optional maxheaderlen specifies the longest length for a non-continued
        header.  When a header line is longer (in characters, with tabs
        expanded to 8 spaces) than maxheaderlen, the header will split as
        defined in the Header class.  Set maxheaderlen to zero to disable
        header wrapping.  The default is 78, as recommended (but not required)
        by RFC 2822.

        The policy keyword specifies a policy object that controls a number of
        aspects of the generator's operation.  If no policy is specified,
        the policy associated with the Message object passed to the
        flatten method is used.

        """

    @overload
    def __init__(
        self,
        outfp: SupportsWrite[str],
        mangle_from_: bool | None = None,
        maxheaderlen: int | None = None,
        *,
        policy: Policy[_MessageT],
    ) -> None: ...
    def write(self, s: str) -> None: ...
    def flatten(self, msg: _MessageT, unixfrom: bool = False, linesep: str | None = None) -> None:
        """Print the message object tree rooted at msg to the output file
        specified when the Generator instance was created.

        unixfrom is a flag that forces the printing of a Unix From_ delimiter
        before the first object in the message tree.  If the original message
        has no From_ delimiter, a 'standard' one is crafted.  By default, this
        is False to inhibit the printing of any From_ delimiter.

        Note that for subobjects, no From_ line is printed.

        linesep specifies the characters used to indicate a new line in
        the output.  The default value is determined by the policy specified
        when the Generator instance was created or, if none was specified,
        from the policy associated with the msg.

        """

    def clone(self, fp: SupportsWrite[str]) -> Self:
        """Clone this generator with the exact same options."""

class BytesGenerator(Generator[_MessageT]):
    """Generates a bytes version of a Message object tree.

    Functionally identical to the base Generator except that the output is
    bytes and not string.  When surrogates were used in the input to encode
    bytes, these are decoded back to bytes for output.  If the policy has
    cte_type set to 7bit, then the message is transformed such that the
    non-ASCII bytes are properly content transfer encoded, using the charset
    unknown-8bit.

    The outfp object must accept bytes in its write method.
    """

    @overload
    def __init__(
        self: BytesGenerator[Any],  # The Policy of the message is used.
        outfp: SupportsWrite[bytes],
        mangle_from_: bool | None = None,
        maxheaderlen: int | None = None,
        *,
        policy: None = None,
    ) -> None:
        """Create the generator for message flattening.

        outfp is the output file-like object for writing the message to.  It
        must have a write() method.

        Optional mangle_from_ is a flag that, when True (the default if policy
        is not set), escapes From_ lines in the body of the message by putting
        a '>' in front of them.

        Optional maxheaderlen specifies the longest length for a non-continued
        header.  When a header line is longer (in characters, with tabs
        expanded to 8 spaces) than maxheaderlen, the header will split as
        defined in the Header class.  Set maxheaderlen to zero to disable
        header wrapping.  The default is 78, as recommended (but not required)
        by RFC 2822.

        The policy keyword specifies a policy object that controls a number of
        aspects of the generator's operation.  If no policy is specified,
        the policy associated with the Message object passed to the
        flatten method is used.

        """

    @overload
    def __init__(
        self,
        outfp: SupportsWrite[bytes],
        mangle_from_: bool | None = None,
        maxheaderlen: int | None = None,
        *,
        policy: Policy[_MessageT],
    ) -> None: ...

class DecodedGenerator(Generator[_MessageT]):
    """Generates a text representation of a message.

    Like the Generator base class, except that non-text parts are substituted
    with a format string representing the part.
    """

    @overload
    def __init__(
        self: DecodedGenerator[Any],  # The Policy of the message is used.
        outfp: SupportsWrite[str],
        mangle_from_: bool | None = None,
        maxheaderlen: int | None = None,
        fmt: str | None = None,
        *,
        policy: None = None,
    ) -> None:
        """Like Generator.__init__() except that an additional optional
        argument is allowed.

        Walks through all subparts of a message.  If the subpart is of main
        type 'text', then it prints the decoded payload of the subpart.

        Otherwise, fmt is a format string that is used instead of the message
        payload.  fmt is expanded with the following keywords (in
        %(keyword)s format):

        type       : Full MIME type of the non-text part
        maintype   : Main MIME type of the non-text part
        subtype    : Sub-MIME type of the non-text part
        filename   : Filename of the non-text part
        description: Description associated with the non-text part
        encoding   : Content transfer encoding of the non-text part

        The default value for fmt is None, meaning

        [Non-text (%(type)s) part of message omitted, filename %(filename)s]
        """

    @overload
    def __init__(
        self,
        outfp: SupportsWrite[str],
        mangle_from_: bool | None = None,
        maxheaderlen: int | None = None,
        fmt: str | None = None,
        *,
        policy: Policy[_MessageT],
    ) -> None: ...



"""Python 'undefined' Codec

    This codec will always raise a UnicodeError exception when being
    used. It is intended for use by the site.py file to switch off
    automatic string to Unicode coercion.

Written by Marc-Andre Lemburg (mal@lemburg.com).

(c) Copyright CNRI, All Rights Reserved. NO WARRANTY.

"""

import codecs
from _typeshed import ReadableBuffer

# These return types are just to match the base types. In reality, these always
# raise an error.

class Codec(codecs.Codec):
    def encode(self, input: str, errors: str = "strict") -> tuple[bytes, int]: ...
    def decode(self, input: ReadableBuffer, errors: str = "strict") -> tuple[str, int]: ...

class IncrementalEncoder(codecs.IncrementalEncoder):
    def encode(self, input: str, final: bool = False) -> bytes: ...

class IncrementalDecoder(codecs.IncrementalDecoder):
    def decode(self, input: ReadableBuffer, final: bool = False) -> str: ...

class StreamWriter(Codec, codecs.StreamWriter): ...
class StreamReader(Codec, codecs.StreamReader): ...

def getregentry() -> codecs.CodecInfo: ...
